package br.com.biblioteca.bo;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;

import br.com.biblioteca.dao.DisciplinaDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Disciplina;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.MensagensConstantes;

public class DisciplinaBO {

	private final static Logger logger = Logger.getLogger(LivrosFavoritosBO.class);
	
	private DisciplinaDAO disciplinaDAO = new DisciplinaDAO();

	public DisciplinaBO(DisciplinaDAO disciplinaDAO) {
		this.disciplinaDAO = disciplinaDAO;

	}

	public DisciplinaBO() {

	}

	public int adicionaDisciplina(Disciplina disciplina) throws PersistenciaException, SQLException, ParseException {
		try {
			if (validaDisciplina(disciplina)) {

				int idDisciplina = disciplinaDAO.adicionaDisciplina(disciplina);
				return idDisciplina;
			}
			throw new NegocioException("Discplina com dados invlidos");
		} catch (Exception e) {
			throw new PersistenciaException(
					MensagensConstantes.MSG_ERR_DISCIPLINA_DADOS_INVALIDOS.replace("?", disciplina.getNome()));
		}
	}

	public boolean editaDisciplina(Disciplina disciplina) throws PersistenciaException {
		if (validaDisciplina(disciplina)) {
			return disciplinaDAO.editaDisciplina(disciplina);
		}
		return false;
	}

	public boolean removeDisciplina(int idDisciplina) throws PersistenciaException, SQLException {
		boolean excluido = false;
		try {
			excluido = disciplinaDAO.removeDisciplina(idDisciplina);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return excluido;
	}

	public List<Disciplina> listarDisciplinasByUsuario(Usuario usuario) throws NegocioException {
		List<Disciplina> disciplinas = new ArrayList<>();

		try {
			disciplinas = disciplinaDAO.listarDisciplinasByUsuario(usuario, "");
		} catch (PersistenciaException | SQLException e) {
			logger.error(e.getMessage());
			throw new NegocioException(MensagensConstantes.SYSTEM_GENERIC_ERROR);
		}
		return disciplinas;
	}

	public Disciplina buscaDisciplinaByNome(String nome) throws PersistenciaException, SQLException {
		return disciplinaDAO.BuscaDisciplinabyNome(nome);
	}

	public Disciplina buscaDisciplinaById(int id) throws PersistenciaException, SQLException {
		return disciplinaDAO.buscaDisciplinabyId(id);
	}

	public Disciplina buscaDisciplinaByCodigo(String codigo) throws PersistenciaException, SQLException {
		return disciplinaDAO.buscaDisciplinabyCodigo(codigo);
	}

	public boolean validaDisciplina(Disciplina disciplina) {

		if (disciplina == null || disciplina.getCodigo() == null || disciplina.getNome() == null) {
			return false;
		}
		return true;
	}

	public LinkedList<Disciplina> ListaDisciplinas(String nome) throws NegocioException {
		LinkedList<Disciplina> listdisciplina = null;

		try {
			listdisciplina = disciplinaDAO.ListarDisciplinas(nome);

		} catch (PersistenciaException | SQLException e) {
			e.printStackTrace();
			throw new NegocioException(e);
		}

		return listdisciplina;

	}

	public void setDisciplinaDAO(DisciplinaDAO disciplinaDAO) {

		this.disciplinaDAO = disciplinaDAO;
	}
}
