package br.com.biblioteca.bo;

import java.lang.Object;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.apache.commons.validator.routines.ISBNValidator;
import java.io.Serializable;
import org.apache.commons.validator.routines.checkdigit.EAN13CheckDigit;
import org.apache.commons.validator.routines.checkdigit.ISBN10CheckDigit;
import org.apache.commons.validator.routines.checkdigit.ISBNCheckDigit;
import org.apache.commons.validator.routines.checkdigit.CheckDigitException;

import br.com.biblioteca.command.LivrosFavoritosCommand;
import br.com.biblioteca.dao.LivroDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.util.MensagensConstantes;


public class LivroBO {

	private static final Logger logger = Logger.getLogger(LivroBO.class);
	private LivroDAO livroDAO = new LivroDAO();

	public void setLivroDAO(LivroDAO livroDAO) {
		this.livroDAO = livroDAO;
	}

	public LivroBO() {
	}

	public int cadastraLivro(Livro livro) throws NegocioException {
		int idLivro = 0;
		try {
			if (validaLivro(livro)) {
				idLivro = livroDAO.cadastraLivro(livro);
				return idLivro;
			}
			throw new NegocioException(
					MensagensConstantes.MSG_ERR_LIVRO_DADOS_INVALIDOS.replace("?", livro.getTitulo()));
		} catch (SQLException | PersistenciaException | ParseException e) {
			throw new NegocioException(MensagensConstantes.MSG_ERR_ISBN_JA_CADASTRADO);
		}
	}

	public Livro buscaLivrobyTitulo(String titulo) throws PersistenciaException, SQLException {
		return livroDAO.buscaLivrobyTitulo(titulo);
	}

	public Livro buscaLivrobyId(int idLivro) throws PersistenciaException, SQLException {
		return livroDAO.buscaLivrobyId(idLivro);
	}

	public boolean removeLivro(int idLivro) throws PersistenciaException, SQLException {
		boolean excluido = false;
		try {
			excluido = livroDAO.removeLivro(idLivro);
		} catch (SQLException e) {
			logger.debug(e.getMessage());
		}
		return excluido;
	}

	public boolean editarLivro(Livro livro) throws NegocioException {

		try {
			if (validaLivro(livro)) {
				return livroDAO.editarLivro(livro);
			}
			throw new NegocioException(
					MensagensConstantes.MSG_ERR_LIVRO_DADOS_INVALIDOS.replace("?", livro.getTitulo()));
		} catch (SQLException | PersistenciaException e) {
			throw new NegocioException(e.getMessage());
		}

	}

	public boolean validaLivro(Livro livro) {
		if (livro == null)
			return false;

		if (livro.getTitulo() == null || livro.getTitulo().equals(""))
			return false;

		/*if (livro.getIsbn() != null && !livro.getIsbn().equals("")) {
			if (!livro.getIsbn().matches("[0-9]*")) {
				return false;
			}
			if (!ISBNCheckDigit.ISBN_CHECK_DIGIT.isValid(livro.getIsbn())) {
				return false; // Esse codigo est temporariamente comentado pois incapacibilita o cadastramento de isbn's invlidos para testes.
			}
		}*/

		if (livro.getIdioma() != null && !livro.getIdioma().equals("")) {
			if (livro.getIdioma().matches("[0-9]*")) {
				return false;
			}

		}

		if (livro.getCidadePublicacao() != null && !livro.getCidadePublicacao().equals("")) {
			if (livro.getCidadePublicacao().matches("[0-9]*")) {
				return false;
			}

		}

		return true;
	}

	public List<Livro> listarLivros(String titulo) throws NegocioException {

		List<Livro> listLivros = null;

		try {
			listLivros = livroDAO.listarLivros(titulo);

		} catch (PersistenciaException | SQLException e) {
			e.printStackTrace();
			throw new NegocioException(e);
		}

		return listLivros;
	}

}
