package br.com.biblioteca.bo;

import java.sql.SQLException;
import java.util.List;

import org.apache.log4j.Logger;

import br.com.biblioteca.dao.LivrosFavoritosDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.MensagensConstantes;

public class LivrosFavoritosBO {

	LivrosFavoritosDAO livrosFavoritosDAO;

	private final static Logger logger = Logger.getLogger(LivrosFavoritosBO.class);

	public LivrosFavoritosBO() {
		this.livrosFavoritosDAO = new LivrosFavoritosDAO();
	}

	public void setLivrosFavoritosDAO(LivrosFavoritosDAO livrosFavoritosDAO) {
		this.livrosFavoritosDAO = livrosFavoritosDAO;
	}

	public boolean adicionaLivroFavoritoBO(int idUsuario, int idLivro) throws NegocioException { 
	
		try {			
			livrosFavoritosDAO.adicionarLivroFavoritoDAO(idUsuario, idLivro);
			
		} catch (PersistenciaException | SQLException | ClassNotFoundException e) {
			logger.error(e.getMessage());
			throw new NegocioException(MensagensConstantes.SYSTEM_GENERIC_ERROR);
		}
	
		return true;

	}

	public boolean removerLivro(Usuario usuario, int idLivro) throws NegocioException {

		boolean queryResult = false;
		try {
			queryResult = livrosFavoritosDAO.removerLivro(usuario.getIdUsuario(), idLivro);
		} catch (PersistenciaException | SQLException | ClassNotFoundException e) {
			logger.error(e.getMessage());
			throw new NegocioException(MensagensConstantes.SYSTEM_GENERIC_ERROR);
		}

		return queryResult;
	}

	public List<Livro> buscaLivrosProfessor(int idUsuario) throws NegocioException {

		List<Livro> favoritos;

		try {
			favoritos = this.livrosFavoritosDAO.listarLivrosFavoritos(idUsuario);
		} catch (PersistenciaException | SQLException | ClassNotFoundException e) {
			logger.error(e.getMessage());
			throw new NegocioException(MensagensConstantes.MSG_ERR_LIVRO_JA_CADASTRADO);
		}

		return favoritos;
	}

}
