package br.com.biblioteca.bo;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import br.com.biblioteca.model.PropostaBibliografia;
import org.apache.log4j.Logger;

import br.com.biblioteca.dao.BibliografiaDAO;
import br.com.biblioteca.dao.PropostaBibliografiaDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.BibliografiaBasica;
import br.com.biblioteca.util.MensagensConstantes;

public class PropostaBibliografiaBO {

	private static final Logger logger = Logger.getLogger(PropostaBibliografiaBO.class);
	
	PropostaBibliografiaDAO propostaBibliografiaDAO = new PropostaBibliografiaDAO();

	/**
	 *  O objetivo deste mtodo  retornar a proposta de bibliografia que est ativa de um professor para uma disciplina.
	 * @param idDisciplina
	 * @param idProfessor
	 * @return propostaAtiva
	 * @throws NegocioException
	 * @throws PersistenciaException 
	 */
	public PropostaBibliografia buscaPropostaBibliografiaAtiva(int idDisciplina, int idProfessor) throws NegocioException, PersistenciaException {

		PropostaBibliografia proposta;

		try {
			
			proposta = propostaBibliografiaDAO.buscaPropostaBibliografia(idProfessor, idDisciplina);
			
		} catch (SQLException | ClassNotFoundException e) {
			logger.error(e.getMessage());
			throw new NegocioException(MensagensConstantes.SYSTEM_GENERIC_ERROR);
		}


		return proposta;

	}

	public void addNaBasica(int idProposta, int idLivro) throws ClassNotFoundException, PersistenciaException, SQLException {
		
		propostaBibliografiaDAO.addLivroNaBasica(idProposta, idLivro);
		
	}

	public void addNaComplementar(int idProposta, int idLivro) throws ClassNotFoundException, PersistenciaException, SQLException {
		propostaBibliografiaDAO.addLivroNaComplementar(idProposta, idLivro);
		
	}
	public void removeLivro(int idProposta, int idLivro) throws ClassNotFoundException, SQLException {
		propostaBibliografiaDAO.removerLivroPropostaByID(idProposta, idLivro);
	}

	public void submeterCoordenador(int idProposta, int idCoordenador) throws ClassNotFoundException, SQLException{
		propostaBibliografiaDAO.submeterAoCoordenador(idProposta, idCoordenador);
		
	}
}
