package br.com.biblioteca.command;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import br.com.biblioteca.bo.BibliografiaBO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.model.BibliografiaBasica;
import br.com.biblioteca.model.BibliografiaComplementar;
import br.com.biblioteca.model.BibliografiaHomologada;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.Constantes;
import br.com.biblioteca.util.Paginas;
import br.com.biblioteca.util.Session;



public class BibliografiaDisciplinaCommand implements Command {
	
	private static final Logger logger = Logger.getLogger(BibliografiaDisciplinaCommand.class);
	
	public String execute(HttpServletRequest request) throws NegocioException {
		
		int parametro = Integer.parseInt(request.getParameter(Constantes.ID_DISCIPLINA));
		Usuario user = Session.getUserSession(request.getSession());
		logger.debug("codigo disciplina = " + parametro);
		BibliografiaHomologada bibliografiaHomologada = this.buscaBibliografia(parametro);
		List<Livro>  basica = buscaBibliografiaBasica(bibliografiaHomologada);
		logger.debug("bibliografia basica = " + basica.toString());
		request.setAttribute(Constantes.LISTA_BASICA, basica);
		List<Livro> complementar = buscaBibliografiaComplementar(bibliografiaHomologada);
		logger.debug("bibliografia complementar = " + complementar.toString());
		request.setAttribute(Constantes.LISTA_COMPLEMENTAR, complementar);
		
		return Paginas.BIBLIOGRAFIA_DISCIPLINA.getJSPFile();		
	}
	
	/**
	 * 
	 * @param idDisciplina
	 * @return A lista de bibliografias j homologadas para a disciplina. 
	 * @throws NegocioException
	 */
	private BibliografiaHomologada buscaBibliografia(int idDisciplina) throws NegocioException {
		BibliografiaBO bibliografiaBO = new BibliografiaBO();
		BibliografiaHomologada bibliografiaHomologada = bibliografiaBO.buscaBibliografiaDisciplina(idDisciplina);
		return bibliografiaHomologada;
	}
	
	private List<Livro> buscaBibliografiaBasica(BibliografiaHomologada homologada) {
		List<Livro> livros = new ArrayList<Livro>();
		livros.addAll(homologada.getBasica().getLivros());
		return livros;
	}
	
	private List<Livro> buscaBibliografiaComplementar(BibliografiaHomologada homologada) {
		return new ArrayList<Livro>(homologada.getComplementar().getLivros());
	}
}

