package br.com.biblioteca.command;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import com.mysql.jdbc.StringUtils;

import br.com.biblioteca.bo.AutorBO;
import br.com.biblioteca.bo.LivroBO;
import br.com.biblioteca.bo.LivrosFavoritosBO;
import br.com.biblioteca.dao.AutorDAO;
import br.com.biblioteca.dao.LivroDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Autor;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.Constantes;
import br.com.biblioteca.util.MensagensConstantes;
import br.com.biblioteca.util.Paginas;
import br.com.biblioteca.util.Session;

public class CoordenadorCommand implements Command {
	private static final Logger logger = Logger.getLogger(CoordenadorCommand.class);

	public String execute(HttpServletRequest request) throws NegocioException, PersistenciaException, SQLException {

		String acao = request.getParameter("acao");
		Usuario user = Session.getUserSession(request.getSession());

		try {
			if (acao != null) {
				if (Constantes.ACTION_CADASTRAR_NOVO_LIVRO.equals(acao)) {
					Livro livro = retornarLivro(request);
					int idLivro = adicionarNovoLivro(livro);
					int idUsuario = user.getIdUsuario();
					adicionarLivroFavorito(idUsuario, idLivro);
				}
				if (Constantes.ACTION_ADICIONAR_LIVRO_FAVORITO.equals(acao)) {
					int idLivro = Integer.parseInt(request.getParameter(Constantes.ID_LIVRO));
					int idUsuario = user.getIdUsuario();
					adicionarLivroFavorito(idUsuario, idLivro);
					String nomeLivro = (String) request.getSession().getAttribute(Constantes.TEXTO_LIVRO);
					List<Livro> livrosBiblioteca = pesquisarLivros(nomeLivro);
					request.setAttribute(Constantes.LISTA_BIBLIOTECA, livrosBiblioteca);
				}
				if (Constantes.ACTION_PESQUISAR_LIVROS.equals(acao)) {
					String nomeLivro = request.getParameter(Constantes.TEXTO_LIVRO);
					List<Livro> livrosBiblioteca = pesquisarLivros(nomeLivro);
					request.setAttribute(Constantes.LISTA_BIBLIOTECA, livrosBiblioteca);
					request.getSession().setAttribute(Constantes.TEXTO_LIVRO, nomeLivro);
				}
				if (Constantes.ACTION_EDITAR_LIVRO.equals(acao)) {
					Livro livro = retornarLivroEdicao(request);
					editarLivro(livro);
				}
				if (Constantes.ACTION_CADASTRAR_NOVO_AUTOR.equals(acao)) {
					Autor autor = new Autor();
					autor.setNome((String) request.getParameter("autores"));
					cadastraAutor(autor);
				}
				if (Constantes.ACTION_EXCLUIR_LIVRO.equals(acao)) {
					int idLivro = Integer.parseInt(request.getParameter(Constantes.ID_LIVRO));
					boolean result = excluirLivro(user, idLivro);
					if (result)
						request.setAttribute(Constantes.SUCCESS_MSG, MensagensConstantes.SUC_EXLCUSAO_LIVRO_FAVORITO);
					else
						request.setAttribute(Constantes.ERROR_MSG, MensagensConstantes.ERR_EXCLUSAO_LIVRO_FAVORITO);
				}
				if (Constantes.ACTION_LISTAR_AUTORES.equals(acao)) {
					AutorBO autorBO = new AutorBO();
					request.setAttribute(Constantes.ACTION_LISTAR_AUTORES, listarAutores());
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			request.setAttribute(Constantes.ERROR_MSG, e.getMessage());

		} finally {
			List<Livro> listagem = carregarListagem(user);
			request.setAttribute(Constantes.LISTA_FAVORITOS, listagem);

			// carregar autores
			List<Autor> autores = listarAutores();
			String[] nomes = new String[autores.size()];
			for (int i = 0; i < autores.size(); i++) {
				nomes[i] = autores.get(i).getNome();
			}
			request.setAttribute(Constantes.LISTA_AUTORES, nomes);

		}
		return Paginas.LIVROS_FAVORITOS.getJSPFile();
	}

	private List<Livro> carregarListagem(Usuario usuario) throws NegocioException {

		LivrosFavoritosBO livrosFavoritosBO = new LivrosFavoritosBO();
		List<Livro> livrosFavoritosDTO = livrosFavoritosBO.buscaLivrosProfessor(usuario.getIdUsuario());

		return livrosFavoritosDTO;
	}

	private List<Autor> listarAutores() throws PersistenciaException, SQLException {
		AutorBO autorBO = new AutorBO();
		return autorBO.listarAutores();
	}

	private void cadastraAutor(Autor autor) throws PersistenciaException, NegocioException {
		AutorBO autoresBO = new AutorBO();
		int resultado = autoresBO.cadastraAutor(autor);
	}

	public boolean excluirLivro(Usuario user, int idLivro) throws NegocioException {

		LivrosFavoritosBO livrosFavoritosBO = new LivrosFavoritosBO();
		boolean result = livrosFavoritosBO.removerLivro(user, idLivro);

		return result;
	}

	private List<Livro> pesquisarLivros(String nomeLivro) throws NegocioException {
		LivroBO livroBO = new LivroBO();
		List<Livro> livros = livroBO.listarLivros(nomeLivro);
		return livros;
	}

	public boolean editarLivro(Livro livro) throws NegocioException, ParseException {
		LivroBO livroBO = new LivroBO();
		boolean result = livroBO.editarLivro(livro);
		return result;
	}

	public boolean adicionarLivroFavorito(int idUsuario, int idLivro) throws NegocioException {
		LivrosFavoritosBO livrosFavoritosBO = new LivrosFavoritosBO();
		List<Livro> livrosFavoritosDTO = livrosFavoritosBO.buscaLivrosProfessor(idUsuario);
		for (Livro livro : livrosFavoritosDTO) {
			if (livro.getIdLivro() == idLivro) {
				throw new NegocioException(MensagensConstantes.MSG_ERR_LIVRO_JA_CADASTRADO);
			}
		}
		boolean result = livrosFavoritosBO.adicionaLivroFavoritoBO(idUsuario, idLivro);
		return result;
	}

	public int adicionarNovoLivro(Livro livro)
			throws NegocioException, PersistenciaException, SQLException, ParseException {
		LivroBO livroBO = new LivroBO();
		LivroDAO dao = new LivroDAO();
		int idLivro = livroBO.cadastraLivro(livro);
		for (Autor autor : livro.getAutores()) {
			dao.cadastraLivroAutor(idLivro, autor.getIdAutor());
		}
		return idLivro;
	}

	private Livro retornarLivro(HttpServletRequest request) throws PersistenciaException, SQLException, ParseException {
		Livro livro = new Livro();
		livro.setTitulo(request.getParameter("titulo"));
		livro.setEdicao(request.getParameter("edicao"));
		livro.setIsbn(request.getParameter("isbn"));
		livro.setEditora(request.getParameter("editora"));
		if (!request.getParameter("anoPublicacao").equals("")) {
		}
		livro.setAnoPublicacao(1999);
		livro.setIdioma(request.getParameter("idioma"));
		livro.setCidadePublicacao(request.getParameter("cidadePublicacao"));
		livro.setNovoCadastro(true);
		AutorDAO dao = new AutorDAO();
		List<Autor> autores = new ArrayList<>();
		Integer tamMax = 11;
		for (int i = 0; i < tamMax; i++) {
			Autor autor = null;
			String nome = "";
			if (i != 0) {
				nome = request.getParameter("typeId" + i + "");
				if (!StringUtils.isNullOrEmpty(nome)) {
					autor = (Autor) dao.buscaAutorByName(nome);
				}
			} else {
				nome = request.getParameter("typeId");
				if (!StringUtils.isNullOrEmpty(nome)) {
					autor = (Autor) dao.buscaAutorByName(nome);
				}
			}
			if (null != autor && null != autor.getNome())
				autores.add(autor);
			else {
				if (!StringUtils.isNullOrEmpty(nome)) {
					Autor autorD = new Autor(nome);
					int id = dao.cadastraAutor(autorD);
					autorD.setIdAutor(id);
					autores.add(autorD);
				}
			}
		}
		livro.setAutores(autores);
		return livro;
	}

	private Livro retornarLivroEdicao(HttpServletRequest request) {
		Livro livro = new Livro();
		livro.setIdLivro(Integer.parseInt(request.getParameter("idLivro-E")));
		livro.setTitulo(request.getParameter("titulo-E"));
		livro.setEdicao(request.getParameter("edicao-E"));
		livro.setIsbn(request.getParameter("isbn-E"));
		livro.setEditora(request.getParameter("editora-E"));
		if (!request.getParameter("anoPublicacao").equals("")) {
			livro.setAnoPublicacao(Integer.parseInt(request.getParameter("anoPublicacao-E")));
		}
		livro.setAnoPublicacao(1999);
		livro.setIdioma(request.getParameter("idioma-E"));
		livro.setCidadePublicacao(request.getParameter("cidadePublicacao-E"));
		livro.setNovoCadastro(true);
		livro.setAutores(null);
		return livro;
	}
}
