package br.com.biblioteca.command;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import br.com.biblioteca.bo.DisciplinaBO;
import br.com.biblioteca.bo.UsuarioBO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.model.Disciplina;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.Constantes;
import br.com.biblioteca.util.MensagensConstantes;
import br.com.biblioteca.util.Paginas;
import br.com.biblioteca.util.SendMail;
import br.com.biblioteca.util.Session;
import br.com.biblioteca.util.Util;

public class HomeCommand implements Command {

	private static final Logger logger = Logger.getLogger(HomeCommand.class);

	@Override
	public String execute(HttpServletRequest request) throws NegocioException {

		String proximaPagina = "";
		
		String acao = request.getParameter(Constantes.ACTION_PARAM);
		
		boolean loginEfetuado = true;
		
		if (Constantes.ACTION_LOGIN.equals(acao)) {
			loginEfetuado = executaLogin(request);
		} else if (Constantes.ACTION_LOGOUT.equals(acao)) {
			loginEfetuado = !executaLogout(request);
			request.setAttribute(Constantes.SUCCESS_MSG, MensagensConstantes.MSG_INF_LOGOUT);
		}
		
		if (loginEfetuado) {
			proximaPagina = executaHomePage(request);
		} else {
			proximaPagina = Paginas.LOGIN.getJSPFile();
			String login = request.getParameter(Constantes.LOGIN_FORM_EMAIL);
			request.setAttribute(Constantes.LOGIN_FORM_EMAIL, login);
		}

		request.getSession().setAttribute(Constantes.PAGE_CONTEXT, proximaPagina);
		return proximaPagina;
	}

	/**
	 * Busca o usurio contido na sesso, sobre a identificao de
	 * 'usuarioSessao'
	 * 
	 * @param request
	 * @return Usuario contido na sesso HTTP
	 * @see HttpServletRequest
	 */
	public Usuario buscaUsuarioSessao(HttpServletRequest request) {
		return (Usuario) request.getSession().getAttribute("usuarioSessao");
	}

	/**
	 * Vlida um objeto UsuarioDTO buscando seus dados no banco de dados da
	 * aplicao
	 * 
	 * @param usuarioDTO
	 *            Objeto usurio contendo dados a serem vlidados
	 * @return Em caso de usurio valido retorna objeto contendo os dados do
	 *         usurio logado, caso contrrio um objeto null
	 * @throws NegocioException
	 * @see Usuario
	 */
	private Usuario validaUsuario(Usuario usuarioDTO) throws NegocioException {

		Usuario user = null;
		UsuarioBO usuarioBO = new UsuarioBO();
		user = usuarioBO.validaUsuario(usuarioDTO);
		return user;

	}

	private boolean configuraUsuarioSessao(Usuario user, HttpServletRequest request) {

		request.getSession().setAttribute(Constantes.USUARIO_SESSAO, user);
		request.getSession().setAttribute(Constantes.VERSAO_SISTEMA, Util.getVersion());

		return true;
	}

	private Usuario montaUsuarioDTO(HttpServletRequest request) {

		String email = request.getParameter(Constantes.LOGIN_FORM_EMAIL);
		String senha = request.getParameter(Constantes.LOGIN_FORM_SENHA);

		Usuario usuarioDTO = new Usuario();
		usuarioDTO.setEmail(email);
		usuarioDTO.setSenha(senha);
		

		return usuarioDTO;
	}

	/**
	 * Executa as aes requeridas no login da aplicao
	 * 
	 * @param request
	 * @return String contendo a prxima pgina a ser encaminhada pelo servlet
	 * @throws NegocioException
	 */
	private boolean executaLogin(HttpServletRequest request) throws NegocioException {

		request.getSession().setAttribute(Constantes.USUARIO_SESSAO, null);

		Usuario usuarioDTO = montaUsuarioDTO(request);
		Usuario user = validaUsuario(usuarioDTO);

		boolean resultadoLogin = false;

		if (user != null) {
			logger.debug("Usurio " + user.getEmail() + " logado com sucesso!");
			user.setSenha("");
			configuraUsuarioSessao(user, request);
			resultadoLogin = true;
		}

		return resultadoLogin;

	}
	
	private boolean executaLogout(HttpServletRequest request) {

		request.setAttribute(Constantes.USUARIO_SESSAO, null);
		request.getSession().invalidate();
		
		return true;
	}

	private String executaHomePage(HttpServletRequest request) throws NegocioException{

		Usuario user = Session.getUserSession(request.getSession());
		DisciplinaBO disciplinasBO = new DisciplinaBO();
		List<Disciplina> disciplinasDTO = disciplinasBO.listarDisciplinasByUsuario(user);
		request.setAttribute(Constantes.LISTA_DISCIPLINAS, disciplinasDTO);
		//mailSubmissaoProfCoord(request); Teste de comunicacao entre classes
		
		return Paginas.DISCIPLINAS.getJSPFile();	
	}
	
	private void mailSubmissaoProfCoord(HttpServletRequest request) throws NegocioException {
		//e-mail de confirmacao para o professor
		Usuario user = Session.getUserSession(request.getSession());
		SendMail mailP = new SendMail(user.getNome(),user.getEmail()); 
		mailP.setMail(1); //Contexto: envio do professor
		mailP.envio();
		
		//e-mail de notificacao ao coordenador
		Usuario coord = new Usuario();
		SendMail mailC = new SendMail(coord.getNome(), coord.getEmail()); 
		//TODO Catar o e-mail do coordenador (passear pelo sql)
		
	}
}
