package br.com.biblioteca.command;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import br.com.biblioteca.bo.AutorBO;
import br.com.biblioteca.bo.LivroBO;
import br.com.biblioteca.bo.LivrosFavoritosBO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Autor;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.Constantes;
import br.com.biblioteca.util.MensagensConstantes;
import br.com.biblioteca.util.Paginas;
import br.com.biblioteca.util.Session;

public class LivrosFavoritosCommand implements Command {

    private static final Logger logger = Logger.getLogger(LivrosFavoritosCommand.class);

    public String execute(HttpServletRequest request) throws NegocioException, PersistenciaException {

        String acao = request.getParameter("acao");
        Usuario user = Session.getUserSession(request.getSession());

        try {
            if (acao != null) {
                if (Constantes.ACTION_CADASTRAR_NOVO_LIVRO.equals(acao)) {
                    Livro livro = retornarLivro(request);
                    int idLivro = adicionarNovoLivro(livro);
                    int idUsuario = user.getIdUsuario();
                    adicionarLivroFavorito(idUsuario, idLivro);
                }
                if (Constantes.ACTION_ADICIONAR_LIVRO_FAVORITO.equals(acao)) {
                    int idLivro = Integer.parseInt(request.getParameter(Constantes.ID_LIVRO));
                    int idUsuario = user.getIdUsuario();
                    adicionarLivroFavorito(idUsuario, idLivro);
                    String nomeLivro = (String) request.getSession().getAttribute(Constantes.TEXTO_LIVRO);
                    List<Livro> livrosBiblioteca = pesquisarLivros(nomeLivro);
                    request.setAttribute(Constantes.LISTA_BIBLIOTECA, livrosBiblioteca);
                }
                if (Constantes.ACTION_PESQUISAR_LIVROS.equals(acao)) {
                    String nomeLivro = request.getParameter(Constantes.TEXTO_LIVRO);
                    List<Livro> livrosBiblioteca = pesquisarLivros(nomeLivro);
                    request.setAttribute(Constantes.LISTA_BIBLIOTECA, livrosBiblioteca);
                    request.getSession().setAttribute(Constantes.TEXTO_LIVRO, nomeLivro);
                }
                if (Constantes.ACTION_EDITAR_LIVRO.equals(acao)) {
                    Livro livro = retornarLivroEdicao(request);
                    editarLivro(livro);
                }
                if (Constantes.ACTION_CADASTRAR_NOVO_AUTOR.equals(acao)) {
                    Autor autor = new Autor();
                    autor.setNome((String) request.getParameter("nome"));
                    cadastraAutor(autor);
                }
                if (Constantes.ACTION_EXCLUIR_LIVRO.equals(acao)) {
                    int idLivro = Integer.parseInt(request.getParameter(Constantes.ID_LIVRO));
                    boolean result = excluirLivro(user, idLivro);
                    if (result)
                        request.setAttribute(Constantes.SUCCESS_MSG, MensagensConstantes.SUC_EXLCUSAO_LIVRO_FAVORITO);
                    else
                        request.setAttribute(Constantes.ERROR_MSG, MensagensConstantes.ERR_EXCLUSAO_LIVRO_FAVORITO);
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            List<Livro> listagem = carregarListagem(user);
            request.setAttribute(Constantes.LISTA_FAVORITOS, listagem);
        }
        
        return Paginas.LIVROS_FAVORITOS.getJSPFile();
    }

    private void cadastraAutor(Autor autor) throws PersistenciaException, NegocioException {
        AutorBO autoresBO = new AutorBO();
        int resultado = autoresBO.cadastraAutor(autor);
    }

    private Livro retornarLivro(HttpServletRequest request) {
        Livro livro = new Livro();
        livro.setTitulo(request.getParameter("titulo"));
        livro.setEdicao(request.getParameter("edicao"));
        livro.setIsbn(request.getParameter("isbn"));
        livro.setEditora(request.getParameter("editora"));
        if (!request.getParameter("anoPublicacao").equals("")) {
            livro.setAnoPublicacao(Integer.parseInt(request.getParameter("anoPublicacao")));
        }
        livro.setIdioma(request.getParameter("idioma"));
        livro.setCidadePublicacao(request.getParameter("cidadePublicacao"));
        livro.setNovoCadastro(true);
        // livro.setAutores(null);
        return livro;
    }

    private Livro retornarLivroEdicao(HttpServletRequest request) {
        Livro livro = new Livro();

        livro.setIdLivro(Integer.parseInt(request.getParameter("idLivro-E")));
        livro.setTitulo(request.getParameter("titulo-E"));
        livro.setEdicao(request.getParameter("edicao-E"));
        livro.setIsbn(request.getParameter("isbn-E"));
        livro.setEditora(request.getParameter("editora-E"));
        if (!request.getParameter("anoPublicacao").equals("")) {
            livro.setAnoPublicacao(Integer.parseInt(request.getParameter("anoPublicacao-E")));
        }
        livro.setIdioma(request.getParameter("idioma-E"));
        livro.setCidadePublicacao(request.getParameter("cidadePublicacao-E"));
        livro.setNovoCadastro(true);
        // livro.setAutores(null);
        return livro;
    }

    private List<Livro> carregarListagem(Usuario usuario) throws NegocioException {
       LivrosFavoritosBO livrosFavoritosBO = new LivrosFavoritosBO();
        List<Livro> livrosFavoritosDTO = livrosFavoritosBO.buscaLivrosProfessor(usuario.getIdUsuario());

        return livrosFavoritosDTO;
    }

    public boolean excluirLivro(Usuario user, int idLivro) throws NegocioException {

        LivrosFavoritosBO livrosFavoritosBO = new LivrosFavoritosBO();
        boolean result = livrosFavoritosBO.removerLivro(user, idLivro);

        return result;
    }

    private List<Livro> pesquisarLivros(String nomeLivro) throws NegocioException {
        LivroBO livroBO = new LivroBO();
        List<Livro> livros = livroBO.listarLivros(nomeLivro);
        return livros;
    }

    public boolean adicionarLivroFavorito(int idUsuario, int idLivro) throws NegocioException {

        LivrosFavoritosBO livrosFavoritosBO = new LivrosFavoritosBO();

        List<Livro> livrosFavoritosDTO = livrosFavoritosBO.buscaLivrosProfessor(idUsuario);

        for (Livro livro : livrosFavoritosDTO) {
            if (livro.getIdLivro() == idLivro) {
                throw new NegocioException(MensagensConstantes.MSG_ERR_LIVRO_JA_CADASTRADO);
            }
        }

        boolean result = livrosFavoritosBO.adicionaLivroFavoritoBO(idUsuario, idLivro);

        return result;

    }

    public int adicionarNovoLivro(Livro livro) throws NegocioException {
        LivroBO livroBO = new LivroBO();
        int idLivro = livroBO.cadastraLivro(livro);
        return idLivro;
    }

    public boolean editarLivro(Livro livro) throws NegocioException{ // Executa
        LivroBO livroBO = new LivroBO();
        boolean result = livroBO.editarLivro(livro);
        return result;
    }
}
