/**
 * Classe command respons�vel por orquestrar as a��es relativas a proposta de bibliografia.
 *
 */
package br.com.biblioteca.command;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import org.apache.log4j.Logger;
import br.com.biblioteca.bo.BibliografiaBO;
import br.com.biblioteca.bo.LivrosFavoritosBO;
import br.com.biblioteca.bo.PropostaBibliografiaBO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Autor;
import br.com.biblioteca.model.Bibliografia;
import br.com.biblioteca.model.BibliografiaBasica;
import br.com.biblioteca.model.Disciplina;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.model.PropostaBibliografia;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.Constantes;
import br.com.biblioteca.util.MensagensConstantes;
import br.com.biblioteca.util.Paginas;
import br.com.biblioteca.util.SendMail;
import br.com.biblioteca.util.Session;

public class PropostaBibliografiaCommand implements Command{
	private static final Logger logger = Logger.getLogger(PropostaBibliografiaCommand.class);
	PropostaBibliografiaBO bibliografiaBO = new PropostaBibliografiaBO();
	
	/**
	 * Metodo respons�vel por receber uma requisi��o, executa-la e devolver uma String.
	 * @param request
	 * @return String
	 * @throws NegocioException
	 * @throws PersistenciaException 
	 */
	public String execute(HttpServletRequest request) throws NegocioException, PersistenciaException {

		Usuario user = Session.getUserSession(request.getSession());
	 	String acao = request.getParameter("acao");
	 	
	 try {
	  if (acao != null) {
		  
                if (Constantes.ACTION_PROPOSTA_INSERIR_BASICA.equals(acao)) {
                	int idProposta =  Integer.parseInt(request.getParameter(Constantes.ID_PROPOSTA));
                	int idLivro = Integer.parseInt(request.getParameter(Constantes.ID_LIVRO));
                	this.addNaBasica(idProposta, idLivro); 
                	System.out.println("ID DA PROPOSTA "+idProposta);
                	System.out.println("ID DO LIVRO "+idLivro);
                	
                }
                if (Constantes.ACTION_PROPOSTA_INSERIR_COMPLEMENTAR.equals(acao)) {
                	int idProposta =  Integer.parseInt(request.getParameter(Constantes.ID_PROPOSTA));
                	int idLivro = Integer.parseInt(request.getParameter(Constantes.ID_LIVRO));
                	this.addNaComplementar(idProposta, idLivro);                	
                	System.out.println("ID DA PROPOSTA "+idProposta);
                	System.out.println("ID DO LIVRO "+idLivro);
                	
                }
                if (Constantes.ACTION_PROPOSTA_REMOVER.equals(acao)) {
                	int idProposta =  Integer.parseInt(request.getParameter(Constantes.ID_PROPOSTA));
                	int idLivro = Integer.parseInt(request.getParameter(Constantes.ID_LIVRO));
                	this.removeLivro(idProposta, idLivro);               	
                	System.out.println("ID DA PROPOSTA "+idProposta);
                	System.out.println("ID DO LIVRO "+idLivro);
                }
                if (Constantes.ACTION_PROPOSTA_SUBMETER_COORDENADOR.equals(acao)) {
                	int idProposta =  Integer.parseInt(request.getParameter(Constantes.ID_PROPOSTA));
                	int idCoordenador =  Integer.parseInt(request.getParameter(Constantes.ID_COORDENADOR));
                	this.sumbeterCoordenador(idProposta, idCoordenador );    
                	
                	
            		SendMail mailP = new SendMail(user.getNome(),user.getEmail()); 
            		mailP.setMail(1); 
            		mailP.envio();
            		
           
            		SendMail mailC = new SendMail(request.getParameter(Constantes.NOME_COORDENADOR), request.getParameter(Constantes.MAIL_COORDENADOR));
            		mailC.setMail(2); 
            		mailC.envio();

                	
                	
                }
                
                
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {

        	int id_disciplina =  Integer.parseInt(request.getParameter(Constantes.ID_DISCIPLINA));
        	logger.debug("codigo disciplina = " + id_disciplina);
    		PropostaBibliografia proposta = this.buscaProposta(id_disciplina, user.getIdUsuario());
    	    System.out.print("STATUS:  " + proposta.getStatus());
    		request.setAttribute(Constantes.PROPOSTA, proposta);
    		Disciplina disciplina = proposta.getDisciplina();
    		request.setAttribute(Constantes.DISCIPLINA, disciplina);
    		logger.debug("id_disciplina"  + disciplina.getIdDisciplina());
    		List<Livro> basica = buscaBibliografiaBasica(proposta);
    		logger.debug("bibliografia basica = " + basica.toString());
    		request.setAttribute(Constantes.LISTA_BASICA_PROPOSTA, basica);
    		List<Livro> complementar = buscaBibliografiaComplementar(proposta);
    		logger.debug("bibliografia complementar = " + complementar.toString());
    		request.setAttribute(Constantes.LISTA_COMPLEMENTAR_PROPOSTA, complementar);
    		List<Livro> listagem = listaFavoritos(user, proposta);
    		request.setAttribute(Constantes.LISTA_FAVORITOS_PROPOSTA, listagem);
    		
        }
	 
	 return Paginas.PROPOSTA_BIBLIOGRAFIA.getJSPFile();    	
 	
       
    	
    }

	private void sumbeterCoordenador(int idProposta, int idDisciplina) throws ClassNotFoundException, SQLException {
		bibliografiaBO.submeterCoordenador(idProposta, idDisciplina);
		
	}

	private void addNaComplementar(int idProposta, int idLivro) throws ClassNotFoundException, PersistenciaException, SQLException {
		bibliografiaBO.addNaComplementar(idProposta, idLivro);
	}

	private PropostaBibliografia buscaProposta(int idDisciplina, int idProfessor) throws NegocioException, PersistenciaException {
		PropostaBibliografia proposta = bibliografiaBO.buscaPropostaBibliografiaAtiva(idDisciplina, idProfessor);
		return proposta;
	}
	/**
	 * Este mtodo deve retornar uma lista somente com os livros que no esto sendo utilizados
	 * na proposta vigente atual.
	 */
	private   List<Livro> listaFavoritos(Usuario usuario, PropostaBibliografia proposta) throws NegocioException {
		LivrosFavoritosBO livrosFavoritosBO = new LivrosFavoritosBO();
		ArrayList<Livro> livrosFavoritosDTO = new ArrayList<Livro>();
		livrosFavoritosDTO.addAll(livrosFavoritosBO.buscaLivrosProfessor(usuario.getIdUsuario()));
		ArrayList<Livro> cleanFavoritos = new ArrayList<Livro>();		
		for(Livro l : livrosFavoritosDTO) {
			if(proposta.verificarTipoBibliografiaPorIdLivro(l.getIdLivro()) == -1) {
				cleanFavoritos.add(l);
			}
		}
		return cleanFavoritos;
	}
	private void addNaBasica(int idProposta, int idLivro) throws ClassNotFoundException, PersistenciaException, SQLException {
		bibliografiaBO.addNaBasica(idProposta, idLivro);
		
	}

	private List<Livro> buscaBibliografiaBasica(PropostaBibliografia proposta) {
		List<Livro> basica = new ArrayList<Livro>();
		basica.addAll(proposta.getBasica().getLivros());
		return basica;
	}

	private List<Livro> buscaBibliografiaComplementar(PropostaBibliografia proposta) {
		List<Livro> complementar = new ArrayList<Livro>();
		complementar.addAll(proposta.getComplementar().getLivros());
		return complementar;
	}	
	public void removeLivro(int idProposta, int idLivro) throws ClassNotFoundException, SQLException {
		bibliografiaBO.removeLivro(idProposta, idLivro);
	}
	
}
