package br.com.biblioteca.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import com.mysql.jdbc.Statement;

import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Autor;
import br.com.biblioteca.util.ConexaoUtil;
import br.com.biblioteca.util.MensagensConstantes;

public class AutorDAO {
	
	public int cadastraAutor(Autor autor) throws PersistenciaException, SQLException, ParseException {
		Connection conexao = null;
		int idAutor = 0; 
		
		try {					
				conexao = ConexaoUtil.getConexao();
				StringBuilder sql = new StringBuilder();
				sql.append("insert into tb_autores(nome)");
				sql.append("values(?)");			
				
				PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
				statement.setString(1, autor.getNome());
				statement.executeUpdate();
				
				ResultSet resultset = statement.getGeneratedKeys();
				if (resultset.first()) {
					idAutor = resultset.getInt(1);
				}
				return idAutor;	
			} catch (ClassNotFoundException | SQLException e) {
				throw new PersistenciaException(
						
						MensagensConstantes.MSG_ERR_AUTOR_JA_EXISTENTE.replace("?", autor.getNome()));
			} finally {
				conexao.close();
			}				
	}
	
	public boolean editaAutor(Autor autor) throws PersistenciaException, SQLException, ParseException {
		boolean update = false;
		Connection conexao = null;
		try{
			conexao = ConexaoUtil.getConexao();
			

			StringBuilder sql = new StringBuilder();
			sql.append("update tb_autores set ");
			sql.append("nome = ?, ");
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, autor.getNome());
			update = true;									
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {			
				conexao.close();
		}
		return update;						
	}
	
	public Autor buscaAutorById(int idAutor) throws PersistenciaException, SQLException {
		Autor autor = new Autor();				
		Connection conexao = null;
		
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select nome from tb_autores ");
			sql.append("where id_autor = ?");
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setInt(1, idAutor);

			ResultSet resultset = statement.executeQuery();
			if (resultset.next()) {
				autor.setNome(resultset.getString("nome"));
			}
			
			return autor;
			} catch (ClassNotFoundException | SQLException e) {
				throw new PersistenciaException("Error");
			} finally {
				conexao.close();
			}				
		}
	
	public Autor buscaAutorByName(String nome) throws PersistenciaException, SQLException {
		Autor autor = new Autor();				
		Connection conexao = null;
		
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select nome, id_autor from tb_autores ");
			sql.append("where nome = ?");
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, nome);

			ResultSet resultset = statement.executeQuery();
			if (resultset.next()) {
				autor.setNome(resultset.getString("nome"));
				autor.setIdAutor(resultset.getInt("id_autor"));
			}
			
			return autor;
			} catch (ClassNotFoundException | SQLException e) {
				throw new PersistenciaException("Error");
			} finally {
				conexao.close();
			}				
		}
	
	public List<Autor> listarAutores() throws PersistenciaException, SQLException {
		List<Autor> autores =  new ArrayList<>();				
		Connection conexao = null;
		
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select * from tb_autores");
			
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
		

			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {
				Autor autor = new Autor();
			
				autor.setNome(resultset.getString("nome"));
				
				autores.add(autor);
				
			}
			
			return autores;
			} catch (ClassNotFoundException | SQLException e) {
				throw new PersistenciaException("Error");
			} finally {
				conexao.close();
			}				
		}
}