/**
 * A funo desta classe  prover o acesso ao banco de dados e retornar os objetos (bilbliografia) e excees de l oriundos.
 */

package br.com.biblioteca.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import br.com.biblioteca.model.Autor;
import br.com.biblioteca.model.BibliografiaBasica;
import br.com.biblioteca.model.BibliografiaHomologada;
import br.com.biblioteca.model.Disciplina;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.model.PropostaBibliografia;
import br.com.biblioteca.util.ConexaoUtil;

public class BibliografiaDAO {

	/**
	 * Este metdo tem por funo retornar ao Business Object (BO) a bibliografia que no momento da consulta est homologada para a disciplina recebida por parmertro.
	 * @param idDisciplina
	 * @return BibliografiaHomologada
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 * 
	 * Consulta SQL: 
	 * SELECT b.id_bibliografia, b.id_disciplina, b.versao, b.status,  b.observacao, 
	 * c.tipo_bibliografia, c.situacao, l.id_livro, l.titulo, l.edicao, l.isbn, l.editora, l.ano_publicacao, l.idioma, l.cidade_publicacao, 
	 * a.id_autor, a.nome
	 * FROM tb_bibliografias b
	 * INNER JOIN tb_bibliografia_livro c
	 * ON b.id_bibliografia = c.id_bibliografia
	 * INNER JOIN tb_livro l
	 * ON c.id_livro = l.id_livro
	 * INNER JOIN tb_autores_livro al
	 * ON l.id_livro = al.id_livro
	 * INNER JOIN tb_autores a
	 * ON al.id_autor = a.id_autor
	 * WHERE b.status = 0
	 * AND b.flg_registro_vigente = 'S'
	 * AND c. situacao = 'A'
	 * AND b.id_disciplina = ?;
	 * 
	 * Observaes: Atributos relativos aos LOGS no esto sendo preenchidos!
	 */
	public BibliografiaHomologada buscaBibliografiaHomologada(int idDisciplina) throws SQLException, ClassNotFoundException {
		Connection conexao = ConexaoUtil.getConexao();
		StringBuilder sql = new StringBuilder();
		sql.append(" SELECT b.id_bibliografia, b.id_disciplina, b.versao, b.status,  b.observacao, ");
		sql.append("c.tipo_bibliografia, c.situacao, l.id_livro, l.titulo, l.edicao, l.isbn, l.editora, l.ano_publicacao, l.idioma, l.cidade_publicacao, ");
		sql.append("a.id_autor, a.nome ");
		sql.append("FROM tb_bibliografias b ");
		sql.append("INNER JOIN tb_bibliografia_livro c ");
		sql.append("ON b.id_bibliografia = c.id_bibliografia ");
		sql.append("INNER JOIN tb_livro l ");
		sql.append("ON c.id_livro = l.id_livro ");
		sql.append("INNER JOIN tb_autores_livro al ");
		sql.append("ON l.id_livro = al.id_livro ");
		sql.append("INNER JOIN tb_autores a ");
		sql.append("ON al.id_autor = a.id_autor ");
		sql.append("WHERE b.status = 10 ");
		sql.append("AND b.flg_registro_vigente = 'S' ");
		sql.append("AND c. situacao = 'A' ");
		sql.append("AND b.id_disciplina = ? ");
		PreparedStatement statement = conexao.prepareStatement(sql.toString());
		statement.setInt(1, idDisciplina);
		ResultSet resultset = statement.executeQuery();
		BibliografiaHomologada homologada = new BibliografiaHomologada();
		while (resultset.next()) {
			int idBibliografia = resultset.getInt("id_bibliografia");
			homologada.setIdBibliografia(idBibliografia);
	     	Disciplina disciplina = new Disciplina();
			disciplina.setIdDisciplina(resultset.getInt("id_disciplina"));
			homologada.setDisciplina(disciplina);
			int idLivro = resultset.getInt("id_livro");
			Livro livro;
			int codClassificacaoBasica = homologada.getBasica().getClassificacao();
			int codClassificacaoComplementar = homologada.getComplementar().getClassificacao();
			if(homologada.verificarTipoBibliografiaPorIdLivro(idLivro) == codClassificacaoBasica){
				Autor autor = new Autor();
				autor.setNome(resultset.getString("nome"));
				homologada.getLivroByIDAndClassificaco(idLivro, codClassificacaoBasica)
				.addAutor(autor);
			} else if (homologada.verificarTipoBibliografiaPorIdLivro(idLivro) == codClassificacaoComplementar){
				Autor autor = new Autor();
				autor.setNome(resultset.getString("nome"));
				homologada.getLivroByIDAndClassificaco(idLivro, codClassificacaoComplementar)
				.addAutor(autor);
			 }else{
				livro = new Livro();
				livro.setIdLivro(idLivro);
				livro.setTitulo(resultset.getString("titulo"));
				livro.setEdicao(resultset.getString("edicao"));
				livro.setIsbn(resultset.getString("isbn"));
				livro.setEditora(resultset.getString("editora"));
				livro.setAnoPublicacao(resultset.getInt("ano_publicacao"));
				livro.setIdioma(resultset.getString("idioma"));
				livro.setCidadePublicacao(resultset.getString("cidade_publicacao"));
				Autor autor = new Autor();
				autor.setNome(resultset.getString("nome"));
				livro.addAutor(autor);
				 if(resultset.getInt("tipo_bibliografia") == codClassificacaoBasica){
					 homologada.addBookInBasica(livro);
				 }else {
					 homologada.addBookInComplementar(livro);
				 }
				 
				 }	
			 }
		return homologada;
		
	}
}