package br.com.biblioteca.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import com.mysql.jdbc.Statement;

import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Autor;
import br.com.biblioteca.model.Curriculo;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.util.ConexaoUtil;
import br.com.biblioteca.util.MensagensConstantes;

public class CurriculoDAO {

	public CurriculoDAO(){
		super();
	}
	
	public boolean adicionarCurriculo(Curriculo curriculo) {
		Connection conexao = null;
		boolean returnResult = false;
		
		try {	
			conexao = ConexaoUtil.getConexao();
			StringBuilder sql = new StringBuilder();
			sql.append("insert into tb_curriculos(codigo, id_curso)"); //TODO: diciplinas
			sql.append("values (?,?)");	
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setString(1, curriculo.getCodigo());
			statement.setInt(2, curriculo.getCurso().getIdCurso());
			statement.executeUpdate();

			ResultSet resultset = statement.getGeneratedKeys();
			if (resultset.first()) {
				returnResult = true;
			}
			
		} catch (SQLException | ClassNotFoundException e) {
		}	
		return returnResult;
	}
	
	public Curriculo buscarCurriculoByCodigo(String codigo){
		Connection conexao = null;
		Curriculo curriculo = new Curriculo();
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			
			sql.append("select tb_curriculos.id_curriculo, tb_curriculos.codigo, tb_cursos.id_curso, tb_cursos.codigo, tb_cursos.nome, tb_faculdades.id_faculdade, tb_faculdades.nome, tb_disciplinas.id_disciplina, tb_disciplinas.codigo, tb_disciplinas.nome");
			sql.append("from tb_curriculos");
			sql.append("inner join tb_cursos on tb_curriculos.id_curso = tb_cursos.id_curso");
			sql.append("inner join tb_faculdades on tb_cursos.id_faculdade = tb_faculdades.id_faculdade");
			sql.append("inner join tb_curriculo_disciplina on tb_curriculos.id_curriculo = tb_curriculo_disciplina.id_curriculo");
			sql.append("inner join tb_disciplinas on tb_curriculo_disciplina.id_disciplina = tb_disciplinas.id_disciplina");
			sql.append("where codigo = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, codigo);
			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {				
				curriculo.setIdCurriculo(resultset.getInt("codigo"));
				curriculo.setCodigo(codigo);						
			}				
		} catch (SQLException | ClassNotFoundException e) {
		}	
		return curriculo;
	}
	
	public Curriculo buscarCurriculoById(int idCurriculo){
		Connection conexao = null;
		Curriculo curriculo = new Curriculo();
		try {
			conexao = ConexaoUtil.getConexao();
			
			StringBuilder sql = new StringBuilder();
			sql.append("select codigo, id_curso from tb_curriculos"
					+ "inner join tb_cursos on tb_curriculos.id_curso = tb_cursos.id_curso");
			sql.append("where id_curriculo = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setInt(1, idCurriculo);
			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {				
				curriculo.setCodigo(resultset.getString("codigo"));
				curriculo.setIdCurriculo(idCurriculo);						
			}				
		} catch(Exception e) {
		}
		return curriculo;
	}
	
	public boolean removerCurriculo(Curriculo curriculo) throws PersistenciaException, SQLException {
 		boolean removido = false;
 		Connection conexao = null;
 		try {
 			conexao = ConexaoUtil.getConexao();
 			
 			StringBuilder sql = new StringBuilder();
 			sql.append("DELETE FROM tb_curriculos WHERE id_curriculo = ?");
 			
 			PreparedStatement statement = conexao.prepareStatement(sql.toString());
 			statement.setInt(1, curriculo.getIdCurriculo());
 			statement.execute();
 			removido = true;
 		} catch (ClassNotFoundException | SQLException e) {
 			throw new PersistenciaException(e);
 		} finally {
 			conexao.close();
 		}
 		return removido;
	}
	
	public boolean editarCurriculo(Curriculo curriculo) throws PersistenciaException, SQLException{
 		boolean editado = false;
 		Connection conexao = null;
 		try {
 			conexao = ConexaoUtil.getConexao();
 			
 			StringBuilder sql = new StringBuilder();
 			sql.append("update tb_curriculos set ");
 			sql.append("codigo = ?, ");
 			sql.append("id_curso = ?, ");
 			sql.append("where id_curriculo = ?");
 
 			PreparedStatement statement = conexao.prepareStatement(sql.toString());
 			statement.setString(1, curriculo.getCodigo());
 			statement.setInt(2, curriculo.getCurso().getIdCurso());
 			statement.setInt(3, curriculo.getIdCurriculo());
 			statement.execute();
 			editado = true;
 		} catch (ClassNotFoundException | SQLException e) {
 			throw new PersistenciaException(e);
 		} finally {
 			conexao.close();
 		}
 		return editado;
	}
	
	
}
