package br.com.biblioteca.dao;

import java.sql.Connection;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.LinkedList;

import com.mysql.jdbc.Statement;

import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Curso;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.util.ConexaoUtil;
import br.com.biblioteca.util.MensagensConstantes;
import br.com.biblioteca.model.Faculdade;

public class CursoDAO {
	
	private LinkedList<Curso> listaCursos;
	
 public CursoDAO(){
	 listaCursos = new LinkedList();
 }
 
 public int adicionarCurso(Curso curso) throws PersistenciaException, SQLException, ParseException{
	 Connection conexao = null;
	 try{
		 int idCurso = 0;
		conexao = ConexaoUtil.getConexao();

		StringBuilder sql = new StringBuilder();
		sql.append("insert into tb_cursos(codigo, nome, id_faculdade)");
		sql.append("values (?,?,?)");
		
		PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
		statement.setString(1, curso.getCodigo());
		statement.setString(2, curso.getNome());
		statement.setInt(3, curso.getFaculdade().getIdFaculdade());
		
		statement.executeUpdate();

		ResultSet resultset = statement.getGeneratedKeys();
		if (resultset.first()) {
			idCurso = resultset.getInt(1);
		}
		return idCurso;
		 
		 
	 }catch(ClassNotFoundException | SQLException e){
		 //trocar mensagem
		 throw new PersistenciaException(
					MensagensConstantes.MSG_ERR_LIVRO_JA_EXISTENTE.replace("?", curso.getNome()));
	 }finally{
			conexao.close();
	 }
	 
 }
 
 public Curso BuscarCursoPorNome(String nome) throws PersistenciaException, SQLException{
	 
	 Curso curso = null;

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select f.id_faculdade, f.nome, c.id_curso, c.nome, c.codigo, c.id_faculdade from tb_cursos c inner join tb_faculdades f on(c.id_faculdade = f.id_faculdade)");
			sql.append("where c.nome = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setString(1, nome);

			ResultSet resultset = statement.executeQuery();
			if (resultset.next()) {
				Faculdade faculdade = new Faculdade(resultset.getString("f.nome"));
				faculdade.setIdFaculdade(resultset.getInt("f.id_faculdade"));
				
				curso = new Curso();
				curso.setIdCurso(resultset.getInt("id_curso"));
				curso.setCodigo(resultset.getString("codigo"));
				curso.setNome(resultset.getString("nome"));
				
				curso.setFaculdade(faculdade);	
			}
			return curso;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
	 
 }
 
 public Curso BuscarCursoPorId(int id) throws PersistenciaException, SQLException{
	 Curso curso = null;

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select f.id_faculdade, f.nome, c.id_curso, c.nome, c.codigo, c.id_faculdade from tb_cursos c inner join tb_faculdades f on(c.id_faculdade = f.id_faculdade)");
			sql.append("where c.id_curso = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setInt(1, id);
			 
			ResultSet resultset = statement.executeQuery();
			if (resultset.next()) {
				String nomeFaculdade = resultset.getString("f.nome");
				Faculdade faculdade = new Faculdade(nomeFaculdade);
				faculdade.setIdFaculdade(resultset.getInt("f.id_faculdade"));
				
				curso = new Curso();
				curso.setIdCurso(resultset.getInt("id_curso"));
				curso.setCodigo(resultset.getString("codigo"));
				curso.setNome(resultset.getString("nome"));
				
				curso.setFaculdade(faculdade);	
			}
			return curso;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
 }
 
 public boolean editarCurso(Curso curso) throws PersistenciaException{
	 boolean returnUpdate = false;
		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();
		

			StringBuilder sql = new StringBuilder();
			sql.append("update tb_curso set ");
			sql.append("codigo = ?, ");
			sql.append("nome = ?, ");
			sql.append("id_faculdade = ?, ");
			sql.append("where id_curso = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, curso.getCodigo());
			statement.setString(2, curso.getNome());
			statement.setInt(3, curso.getFaculdade().getIdFaculdade());
			statement.setInt(4, curso.getIdCurso());
			
			statement.execute();
			returnUpdate = true;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			try {
				conexao.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return returnUpdate;
 }
 
 public boolean removerCurso(int id_curso) throws PersistenciaException, SQLException{
	 boolean removido = false;
		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();
			
			StringBuilder sql = new StringBuilder();
			sql.append("DELETE FROM tb_curso WHERE id_curso = ?");
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setInt(1, id_curso);
			statement.execute();
			removido = true;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			conexao.close();
		}
		
		return removido;
 }
 
}
