package br.com.biblioteca.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.List;

import com.mysql.jdbc.Statement;

import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Disciplina;
import br.com.biblioteca.util.ConexaoUtil;
import br.com.biblioteca.util.MensagensConstantes;
import br.com.biblioteca.model.Usuario;
 
public class DisciplinaDAO {

private LinkedList<Disciplina> listaDisciplinas;
	
	public DisciplinaDAO() {
		listaDisciplinas = new LinkedList<>();
	}

	public int adicionaDisciplina(Disciplina disciplina) throws PersistenciaException, SQLException, ParseException {
		Connection conexao = null;
		try {
			int idDisciplina = 0;
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("insert into tb_disciplinas(codigo, nome)");
			sql.append("values (?,?)");

			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setString(1, disciplina.getCodigo());
			statement.setString(2, disciplina.getNome());
			statement.setInt(3, disciplina.getIdDisciplina());

			statement.executeUpdate();

			ResultSet resultset = statement.getGeneratedKeys();
			if (resultset.first()) {
				idDisciplina = resultset.getInt(1);
			}
			return idDisciplina;

		} catch (ClassNotFoundException | SQLException e) {

			throw new PersistenciaException(
					MensagensConstantes.MSG_ERR_DISCIPLINA_JA_EXISTENTE.replace("?", disciplina.getNome()));
		} finally {
			conexao.close();
		}

	}

	public Disciplina BuscaDisciplinabyNome(String nome) throws PersistenciaException, SQLException {

		Disciplina disciplina = new Disciplina();

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select id_disciplina, codigo, nome from tb_disciplinas ");
			sql.append("where nome = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setString(1, nome);

			ResultSet resultset = statement.executeQuery();
			if (resultset.next()) {
				disciplina = new Disciplina();
				disciplina.setIdDisciplina(resultset.getInt("id_disciplina"));
				disciplina.setCodigo(resultset.getString("codigo"));
				disciplina.setNome(resultset.getString("nome"));
			}
			return disciplina;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}

	}
	
	public List<Disciplina> listarDisciplinasByUsuario(Usuario usuario, String tipoUsuario) throws PersistenciaException, SQLException {

		List<Disciplina> disciplinas = new LinkedList<>();

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			
			sql.append("select tb_disciplinas.id_disciplina, tb_disciplinas.codigo, tb_disciplinas.nome, tb_disciplinas.data_alteracao, tb_disciplinas.status ");
			sql.append("from tb_professores ");
			sql.append("inner join tb_disciplinas_professores on tb_professores.id_professor = tb_disciplinas_professores.id_professor ");
			sql.append("inner join tb_disciplinas on tb_disciplinas_professores.id_disciplina = tb_disciplinas.id_disciplina ");
			sql.append("where tb_professores.id_professor = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setInt(1, usuario.getIdUsuario());

			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {
				Disciplina disciplina = new Disciplina();
				disciplina.setIdDisciplina(resultset.getInt("id_disciplina"));
				disciplina.setCodigo(resultset.getString("codigo"));
				disciplina.setNome(resultset.getString("nome"));
				disciplina.setData_alteracao(resultset.getDate("data_alteracao"));
				disciplina.setStatus(resultset.getString("status"));
				disciplinas.add(disciplina);
			}
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
		return disciplinas;
	}

	public Disciplina buscaDisciplinabyCodigo(String codigo) throws PersistenciaException, SQLException {
		Disciplina disciplina = new Disciplina();

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select id_disciplina, codigo, nome from tb_disciplinas");
			sql.append("where codigo = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setString(1, codigo);

			ResultSet resultset = statement.executeQuery();
			if (resultset.next()) {
				disciplina = new Disciplina();
				disciplina.setIdDisciplina(resultset.getInt("id_disciplina"));
				disciplina.setCodigo(resultset.getString("codigo"));
				disciplina.setNome(resultset.getString("nome"));
			}
			return disciplina;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
	}

	public Disciplina buscaDisciplinabyId(int id) throws PersistenciaException, SQLException {
		Disciplina disciplina = new Disciplina();

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select id_disciplina, codigo, nome from tb_disciplinas");
			sql.append("where id_disciplina = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setInt(1, id);

			ResultSet resultset = statement.executeQuery();
			if (resultset.next()) {
				disciplina = new Disciplina();
				disciplina.setIdDisciplina(resultset.getInt("id_disciplina"));
				disciplina.setCodigo(resultset.getString("codigo"));
				disciplina.setNome(resultset.getString("nome"));
			}
			return disciplina;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
	}

	public boolean removeDisciplina(int idDisciplina) throws PersistenciaException, SQLException {
		boolean removido = false;
		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("DELETE FROM tb_disciplinas WHERE id_disciplina = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setInt(1, idDisciplina);
			statement.execute();
			removido = true;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			conexao.close();
		}
System.out.println();
		return removido;
	}
	

	public boolean editaDisciplina(Disciplina disciplina) throws PersistenciaException {
		boolean returnUpdate = false;
		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("update tb_disciplinas set ");
			sql.append("codigo = ?, ");
			sql.append("nome = ?, ");
			sql.append("where id_disciplina= ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, disciplina.getCodigo());
			statement.setString(2, disciplina.getNome());
			statement.setInt(3, disciplina.getIdDisciplina());

			statement.execute();
			returnUpdate = true;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			try {
				conexao.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return returnUpdate;
	}
	
	public LinkedList<Disciplina> ListarDisciplinas(String nome) throws PersistenciaException, SQLException{
		Connection conexao = null;
		Disciplina disciplina = new Disciplina();
		
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select id_disciplina, nome, codigo"
		     + "from tb_disciplinas");
			sql.append("where nome = ?");
		
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, nome);
			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {				
				disciplina.setNome(resultset.getString("nome"));
				disciplina.setCodigo(resultset.getString("codigo"));
				disciplina.setIdDisciplina(resultset.getInt("id_disciplina"));	
				listaDisciplinas.add(disciplina);
			}

		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			conexao.close();
		}
		
		
		
		return listaDisciplinas;
		
	}

}