package br.com.biblioteca.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.mysql.jdbc.Statement;

import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Autor;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.util.ConexaoUtil;
import br.com.biblioteca.util.MensagensConstantes;

public class LivroDAO {
	
	//metodos que sero necessrios: editaLivro, buscaLivrosPorTitulo, buscaLivroPorId, cadastraLivro, remove Livro. 
	
	public LivroDAO(){
	}
		
	public void cadastraLivroAutor(int livro, int autor) throws PersistenciaException, SQLException, ParseException {
		Connection conexao = null;
		
		try {		
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("insert into tb_autores_livro(id_livro, id_autor)");
			sql.append("values (?,?)");
			
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.SUCCESS_NO_INFO);
			statement.setInt(1, livro);
			statement.setInt(2, autor);
			statement.executeUpdate();

		} catch (ClassNotFoundException | SQLException e) {
			e.printStackTrace();
		} finally {
			conexao.close();
		}	
	
	}
	public int cadastraLivro(Livro livro) throws PersistenciaException, SQLException, ParseException {
		Connection conexao = null;
		
		try {		
			int idLivro = 0;
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("insert into tb_livro(titulo, edicao, isbn, editora, ano_publicacao, idioma, cidade_publicacao, novo_cadastro)");
			sql.append("values (?,?,?,?,?,?,?,?)");
			
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
			statement.setString(1, livro.getTitulo());
			statement.setString(2, livro.getEdicao());
			statement.setString(3, livro.getIsbn());
			statement.setString(4, livro.getEditora());
			statement.setInt(5, livro.getAnoPublicacao());
			statement.setString(6, livro.getIdioma());
			statement.setString(7, livro.getCidadePublicacao());
			statement.setBoolean(8, livro.isNovoCadastro());
			statement.executeUpdate();

			ResultSet resultset = statement.getGeneratedKeys();
			if (resultset.first()) {
				idLivro = resultset.getInt(1);
			}				
			
			return idLivro;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(
					MensagensConstantes.MSG_ERR_LIVRO_JA_EXISTENTE.replace("?", livro.getTitulo()));
		} finally {
			conexao.close();
		}	
	}
	
	public Livro buscaLivrobyTitulo(String titulo) throws PersistenciaException, SQLException {
		Livro livro = new Livro();
		ArrayList<Autor> autores = new ArrayList<>();

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select titulo, edicao, isbn, editora, ano_publicacao,"
					+ " idioma, cidade_publicacao, novo_cadastro, nome from tb_livro"
					+ "inner join tb_autores_livro on tb_livro.id_livro = tb_autores_livro.id_livro "
					+ "inner join tb_autores on tb_autores_livro.id_autor = tb_autores.id_autor");
			sql.append("where tb_livro.titulo = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, titulo);

			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {				
				livro.setTitulo(resultset.getString("titulo"));
				livro.setEdicao(resultset.getString("edicao"));
				livro.setIsbn(resultset.getString("isbn"));	
				livro.setEditora(resultset.getString("editora"));
				livro.setAnoPublicacao(resultset.getInt("ano_publicacao"));
				livro.setIdioma(resultset.getString("idioma"));
				livro.setCidadePublicacao(resultset.getString("cidade_publicacao"));
				boolean novo = (resultset.getInt("novo_cadastro") == 1);
				livro.setNovoCadastro(novo);
				Autor autor = new Autor();
				autor.setNome(resultset.getString("nome"));
				autores.add(autor);								
			}			
			livro.setAutores(autores);
			
			return livro;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
	}	
	
	public Livro buscaLivrobyNomeAutor(String nome) throws PersistenciaException, SQLException {
		Livro livro = new Livro();
		ArrayList<Autor> autores = new ArrayList<>();

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select titulo, edicao, isbn, editora, ano_publicacao,"
					+ " idioma, cidade_publicacao, novo_cadastro, nome from tb_livro"
					+ "inner join tb_autores_livro on tb_livro.id_livro = tb_autores_livro.id_livro "
					+ "inner join tb_autores on tb_autores_livro.id_autor = tb_autores.id_autor");
			sql.append("where tb_autores.nome = ?");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, nome);

			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {				
				livro.setTitulo(resultset.getString("titulo"));
				livro.setEdicao(resultset.getString("edicao"));
				livro.setIsbn(resultset.getString("isbn"));	
				livro.setEditora(resultset.getString("editora"));
				livro.setAnoPublicacao(resultset.getInt("ano_publicacao"));
				livro.setIdioma(resultset.getString("idioma"));
				livro.setCidadePublicacao(resultset.getString("cidade_publicacao"));
				livro.setNovoCadastro(resultset.getBoolean("novo_cadastro"));
				Autor autor = new Autor();
				autor.setNome(resultset.getString("nome"));
				autores.add(autor);								
			}			
			livro.setAutores(autores);
			
			return livro;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
	}
	
	
	public Livro buscaLivrobyId(int idLivro) throws PersistenciaException, SQLException {
		Livro livro = new Livro();
		ArrayList<Autor> autores = new ArrayList<>();

		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select titulo, edicao, isbn, editora, ano_publicacao,"
					+ " idioma, cidade_publicacao, novo_cadastro, nome from tb_livro"
					+ "inner join tb_autores_livro on tb_livro.id_livro = tb_autores_livro.id_livro "
					+ "inner join tb_autores on tb_autores_livro.id_autor = tb_autores.id_autor");
			sql.append("where tb_livro.id_livro = ?");


			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setInt(1, idLivro);

			ResultSet resultset = statement.executeQuery();
			while (resultset.next()) {				
				livro.setTitulo(resultset.getString("titulo"));
				livro.setEdicao(resultset.getString("edicao"));
				livro.setIsbn(resultset.getString("isbn"));	
				livro.setEditora(resultset.getString("editora"));
				livro.setAnoPublicacao(resultset.getInt("ano_publicacao"));
				livro.setIdioma(resultset.getString("idioma"));
				livro.setCidadePublicacao(resultset.getString("cidade_publicacao"));
				livro.setNovoCadastro(resultset.getBoolean("novo_cadastro"));
				Autor autor = new Autor();
				autor.setNome(resultset.getString("nome"));
				autores.add(autor);								
			}				
			livro.setAutores(autores);			
			
			return livro;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException("Error");

		} finally {
			conexao.close();
		}
	}
	
				
	
	public boolean removeLivro(int idLivro) throws PersistenciaException, SQLException {
		boolean removido = false;
		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();
			
			StringBuilder sql = new StringBuilder();
			sql.append("DELETE * FROM tb_livro AND tb_autores_livro WHERE ID_ACERVO = ?");
			
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setInt(1, idLivro);
			statement.execute();
			removido = true;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			conexao.close();
		}		
		return removido;
	}
	
	public boolean editarLivro(Livro livro) throws PersistenciaException, SQLException {
		boolean returnUpdate = false;
		Connection conexao = null;
		try {
			conexao = ConexaoUtil.getConexao();
		

			StringBuilder sql = new StringBuilder();
			sql.append("update tb_livro set ");
			sql.append("titulo = ?, ");
			sql.append("edicao = ?, ");
			sql.append("isbn = ?, ");
			sql.append("editora = ?, ");
			sql.append("ano_publicacao = ?, ");
			sql.append("idioma = ?, ");
			sql.append("cidade_publicacao = ?, ");
			sql.append("novo_cadastro = ?");
			sql.append("where id_livro = ?");
			

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, livro.getTitulo());
			statement.setString(2, livro.getAutores().toString());
			statement.setString(3, livro.getEdicao());
			statement.setString(4, livro.getIsbn());
			statement.setString(6, livro.getEditora());
			statement.setInt(7, livro.getAnoPublicacao());
			statement.setString(8, livro.getIdioma());
			statement.setString(9, livro.getCidadePublicacao());
			int novo = (livro.isNovoCadastro()) ? 1 : 0;
			statement.setInt(8, novo);
			statement.setInt(9, livro.getIdLivro());
			statement.execute();
			returnUpdate = true;
		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			try {
				conexao.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return returnUpdate;
	}
	
	public List<Livro> listarLivros(String texto) throws PersistenciaException, SQLException, NegocioException {
		Connection conexao = null;
		HashMap<Integer, Livro> livros = new HashMap<>();

		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select tb_livro.id_livro, titulo, edicao, isbn, editora, ano_publicacao,");
			sql.append(" idioma, cidade_publicacao, novo_cadastro, nome from tb_livro");
			sql.append(" inner join tb_autores_livro on tb_livro.id_livro = tb_autores_livro.id_livro");
			sql.append(" inner join tb_autores on tb_autores_livro.id_autor = tb_autores.id_autor");
			sql.append(" where titulo LIKE ? OR tb_autores.nome LIKE ?");
			sql.append("ORDER BY tb_livro.id_livro, tb_autores.nome");

			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			statement.setString(1, "%"+texto+"%");
			statement.setString(2, "%"+texto+"%");
			ResultSet resultSet = statement.executeQuery();			
			
			while (resultSet.next()) {	

				int idLivro = resultSet.getInt("id_livro");
				Livro livro = livros.get(idLivro);

				if (livro == null) {
					livro = new Livro();
					livro.setIdLivro(idLivro);
					livro.setTitulo(resultSet.getString("titulo"));
					livro.setEdicao(resultSet.getString("edicao"));
					livro.setIsbn(resultSet.getString("isbn"));
					livro.setEditora(resultSet.getString("editora"));
					livro.setAnoPublicacao(resultSet.getInt("ano_publicacao"));
					livro.setIdioma(resultSet.getString("idioma"));
					livro.setCidadePublicacao(resultSet.getString("cidade_publicacao"));
				}

				Autor autor = new Autor();
				autor.setNome(resultSet.getString("nome"));

				livro.addAutor(autor);

				livros.put(idLivro, livro);
				
			}

		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			conexao.close();
		}
		List<Livro> livrosBiblioteca = new LinkedList<>();

		for (Map.Entry<Integer, Livro> livro : livros.entrySet()) {
			livrosBiblioteca.add(livro.getValue());
		}

		return livrosBiblioteca;
	}
}
