package br.com.biblioteca.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.mysql.jdbc.Statement;

import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Autor;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.ConexaoUtil;
import br.com.biblioteca.util.MensagensConstantes;

public class LivrosFavoritosDAO {

	private static Logger logger = Logger.getLogger(LivrosFavoritosDAO.class);

	public LivrosFavoritosDAO() {
	}

	
	public boolean adicionarLivroFavoritoDAO(int idUsuario, int idLivro) throws PersistenciaException, SQLException, ClassNotFoundException {
		
		boolean status = false;
		Connection conexao = null;

		conexao = ConexaoUtil.getConexao();

		StringBuilder sql = new StringBuilder();
		sql.append("insert into tb_livro_favorito(id_Livro, id_professor)");
		sql.append("values (?, ? )");

		PreparedStatement statement = conexao.prepareStatement(sql.toString(), Statement.RETURN_GENERATED_KEYS);
		statement.setInt(1, idLivro);
		statement.setInt(2, idUsuario);
		status = statement.execute();

		return status;
	}

	public List<Livro> listarLivrosFavoritos(int idUsuario)
			throws PersistenciaException, SQLException, ClassNotFoundException {

		HashMap<Integer, Livro> livros = new HashMap<>();

		Connection conexao = null;

		conexao = ConexaoUtil.getConexao();

		StringBuilder query = new StringBuilder();
		query.append("SELECT tb_livro.id_livro, tb_livro.titulo, tb_livro.edicao, tb_livro.isbn, tb_livro.editora, ")
				.append("tb_livro.ano_publicacao, tb_livro.idioma, tb_livro.cidade_publicacao, tb_autores.nome, tb_livro.novo_cadastro ")
				.append("FROM tb_usuarios INNER JOIN tb_livro_favorito ON tb_usuarios.id_usuario = tb_livro_favorito.id_professor ")
				.append("INNER JOIN tb_livro ON tb_livro_favorito.id_livro = tb_livro.id_livro ")
				.append("LEFT JOIN tb_autores_livro ON tb_livro.id_livro = tb_autores_livro.id_livro ")
				.append("LEFT JOIN tb_autores ON tb_autores_livro.id_autor = tb_autores.id_autor ")
				.append("WHERE tb_usuarios.id_usuario = ? ").append("ORDER BY tb_livro.id_livro, tb_autores.nome;");

		PreparedStatement statement = conexao.prepareStatement(query.toString());
		statement.setInt(1, idUsuario);

		ResultSet resultSet = statement.executeQuery();
		while (resultSet.next()) {

			int idLivro = resultSet.getInt("id_livro");
			Livro livro = livros.get(idLivro);

			if (livro == null) {
				livro = new Livro();
				livro.setIdLivro(idLivro);
				livro.setTitulo(resultSet.getString("titulo"));
				livro.setEdicao(resultSet.getString("edicao"));
				livro.setIsbn(resultSet.getString("isbn"));
				livro.setEditora(resultSet.getString("editora"));
				livro.setAnoPublicacao(resultSet.getInt("ano_publicacao"));
				livro.setIdioma(resultSet.getString("idioma"));
				livro.setCidadePublicacao(resultSet.getString("cidade_publicacao"));
				boolean novo = (resultSet.getInt("novo_cadastro") == 1);
				livro.setNovoCadastro(novo);
			}

			Autor autor = new Autor();
			autor.setNome(resultSet.getString("nome"));

			livro.addAutor(autor);

			livros.put(idLivro, livro);
		}

		List<Livro> livrosFavoritos = new LinkedList<>();

		for (Map.Entry<Integer, Livro> livro : livros.entrySet()) {
			livrosFavoritos.add(livro.getValue());
		}

		logger.debug(livrosFavoritos.toString());

		return livrosFavoritos;

	}

	public boolean buscaLivroIdFavoritos(Livro livro, Usuario usuario) throws PersistenciaException, SQLException {
		Connection conexao = null;
		boolean ok = false;
		try {
			conexao = ConexaoUtil.getConexao();

			StringBuilder sql = new StringBuilder();
			sql.append("select id_livro from tb_livro_FAVORITOS");
			sql.append("where id_usuario = ? and id_livro = ?");
			PreparedStatement statement = conexao.prepareStatement(sql.toString());
			ResultSet resultset = statement.executeQuery();

			ok = resultset.next();
			statement.setInt(1, usuario.getIdUsuario());
			statement.setInt(2, livro.getIdLivro());

		} catch (ClassNotFoundException | SQLException e) {
			throw new PersistenciaException(e);
		} finally {
			conexao.close();
		}
		return ok;
	}

	public boolean removerLivro(int idUsuario, int idLivro) throws PersistenciaException, SQLException, ClassNotFoundException {

		boolean livroRemovido = false;
		Connection conexao = null;

		conexao = ConexaoUtil.getConexao();

		StringBuilder sql = new StringBuilder();
		sql.append("DELETE FROM tb_livro_favorito WHERE id_professor = ? AND id_livro= ? ");

		PreparedStatement statement = conexao.prepareStatement(sql.toString());
		statement.setInt(1, idUsuario);
		statement.setInt(2, idLivro);
		livroRemovido = !statement.execute();
		System.out.println("chegou aqui ID=" + idUsuario + " e o Status=" + livroRemovido);
		
		return livroRemovido;
	}

}
