
/***
* Scripts para criacao e insersao de dados
* Base Dados Biblioteca
***/

CREATE SCHEMA IF NOT EXISTS biblioteca_e DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci ;
USE biblioteca_e ;

-- -----------------------------------------------------
-- Table tb_livro
-- -----------------------------------------------------
-- DROPTABLE tb_livro ;


CREATE TABLE `tb_livro` (
  `id_livro` int(11) NOT NULL AUTO_INCREMENT,
  `titulo` varchar(100) DEFAULT NULL,
  `edicao` varchar(100) DEFAULT NULL,
  `isbn` varchar(100) DEFAULT NULL,
  `editora` varchar(100) DEFAULT NULL,
  `ano_publicacao` int(11) DEFAULT NULL,
  `idioma` varchar(100) DEFAULT NULL,
  `cidade_publicacao` varchar(100) DEFAULT NULL,
  `novo_cadastro` tinyint(1) DEFAULT NULL,
  PRIMARY KEY (`id_livro`),
  UNIQUE KEY `isbn_UNIQUE` (`isbn`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- -----------------------------------------------------
-- Tabletb_autores
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_autores ;

CREATE TABLE tb_autores (
  id_autor INT NOT NULL AUTO_INCREMENT,
  nome VARCHAR(100) NOT NULL,
  PRIMARY KEY (id_autor)  )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_autores_livro
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_autores_livro ;

CREATE TABLE tb_autores_livro (
  id_livro INT NOT NULL,
  id_autor INT NOT NULL,
  PRIMARY KEY (id_livro, id_autor) ,
  INDEX fk_table1_TB_AUTORES1_idx (id_autor ASC) ,
  CONSTRAINT fk_table1_TB_LIVROS0
    FOREIGN KEY (id_livro)
    REFERENCES tb_livro (id_livro)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_table1_TB_AUTORES10
    FOREIGN KEY (id_autor)
    REFERENCES tb_autores (id_autor)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_faculdades
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_faculdades ;

CREATE TABLE tb_faculdades (
  id_faculdade INT NOT NULL AUTO_INCREMENT,
  nome VARCHAR(100) NULL,
  PRIMARY KEY (id_faculdade) ,
  UNIQUE INDEX nome_UNIQUE (nome ASC)  )
ENGINE = InnoDB;



-- -----------------------------------------------------
-- Tabletb_disciplinas
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_disciplinas ;

CREATE TABLE tb_disciplinas (
  id_disciplina INT NOT NULL AUTO_INCREMENT,
  codigo VARCHAR(100) NULL,
  nome VARCHAR(100) NULL,
  data_alteracao DATETIME NULL,
  status VARCHAR(100) NULL,
  PRIMARY KEY (id_disciplina) ,
  UNIQUE INDEX codigo_UNIQUE (codigo ASC)  )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_usuarios
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_usuarios ;

CREATE TABLE tb_usuarios (
  id_usuario INT NOT NULL AUTO_INCREMENT,
  nome VARCHAR(100) NULL,
  email VARCHAR(100) NULL,
  senha VARCHAR(240) NULL,
  data_inclusao DATETIME NULL,
  PRIMARY KEY (id_usuario)  )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_cursos
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_cursos ;

CREATE TABLE tb_cursos (
  id_curso INT NOT NULL AUTO_INCREMENT,
  codigo VARCHAR(100) NULL,
  nome VARCHAR(100) NULL,
  id_faculdade INT NOT NULL,
  id_coordenador INT NULL,
  PRIMARY KEY (id_curso, id_faculdade) ,
  UNIQUE INDEX codigo_UNIQUE (codigo ASC) ,
  INDEX fk_tb_curso_tb_faculdade1_idx (id_faculdade ASC) ,
  CONSTRAINT fk_tb_curso_tb_faculdade1
    FOREIGN KEY (id_faculdade)
    REFERENCES tb_faculdades (id_faculdade),
  CONSTRAINT `fk_tb_curso_tb_usuario`
  	FOREIGN KEY (`id_coordenador`)
  	REFERENCES `biblioteca_e`.`tb_usuarios` (`id_usuario`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;
-- -----------------------------------------------------
-- Tabletb_professores
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_professores ;

CREATE TABLE tb_professores (
  id_professor INT NOT NULL,
  matricula INT NULL,
  PRIMARY KEY (id_professor) ,
  UNIQUE INDEX matricula_UNIQUE (matricula ASC) ,
  INDEX fk_tb_professores_tb_usuario1_idx (id_professor ASC) ,
  CONSTRAINT fk_tb_professores_tb_usuario1
    FOREIGN KEY (id_professor)
    REFERENCES tb_usuarios (id_usuario)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_disciplinas_professores
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_disciplinas_professores ;

CREATE TABLE tb_disciplinas_professores (
  id_disciplina INT NOT NULL,
  id_professor INT NOT NULL,
  PRIMARY KEY (id_disciplina, id_professor) ,
  INDEX fk_tb_disciplina_professor_tb_disciplina1_idx (id_disciplina ASC) ,
  INDEX fk_tb_disciplinas_professores_tb_professores1_idx (id_professor ASC) ,
  CONSTRAINT fk_tb_disciplina_professor_tb_disciplina1
    FOREIGN KEY (id_disciplina)
    REFERENCES tb_disciplinas (id_disciplina)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_tb_disciplinas_professores_tb_professores1
    FOREIGN KEY (id_professor)
    REFERENCES tb_professores (id_professor)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_tipos_bibliografia
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_tipos_bibliografia ;

CREATE TABLE tb_tipos_bibliografia (
  id_tipo_bibliografia INT NOT NULL AUTO_INCREMENT,
  descricao VARCHAR(100) NULL,
  PRIMARY KEY (id_tipo_bibliografia) ,
  UNIQUE INDEX descricao_UNIQUE (descricao ASC)  )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_bibliografias
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_bibliografias ;

CREATE TABLE tb_bibliografias (
  id_bibliografia INT NOT NULL AUTO_INCREMENT,
  id_tipo_bibliografia INT NOT NULL,
  id_professor INT NOT NULL,
  id_disciplina INT NOT NULL,
  id_livro INT NOT NULL,
  classificacao INT NOT NULL, -- 0 = B�sica, 1 = Complementar
  PRIMARY KEY (id_bibliografia, id_tipo_bibliografia, id_professor, id_disciplina, id_livro) ,
  INDEX fk_tb_bibliografias_tb_tipos_bibliografia1_idx (id_tipo_bibliografia ASC) ,
  INDEX fk_tb_bibliografias_tb_professores1_idx (id_professor ASC) ,
  INDEX fk_tb_bibliografias_tb_disciplinas1_idx (id_disciplina ASC) ,
  CONSTRAINT fk_tb_bibliografias_tb_tipos_bibliografia1
    FOREIGN KEY (id_tipo_bibliografia)
    REFERENCES tb_tipos_bibliografia (id_tipo_bibliografia)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_tb_bibliografias_tb_professores1
    FOREIGN KEY (id_professor)
    REFERENCES tb_professores (id_professor)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_tb_bibliografias_tb_disciplinas1
    FOREIGN KEY (id_disciplina)
    REFERENCES tb_disciplinas (id_disciplina)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_curriculos
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_curriculos ;

CREATE TABLE tb_curriculos (
  id_curriculo INT NOT NULL AUTO_INCREMENT,
  codigo VARCHAR(100) NULL,
  id_curso INT NOT NULL,
  PRIMARY KEY (id_curriculo, id_curso) ,
  INDEX fk_tb_curriculos_tb_cursos1_idx (id_curso ASC) ,
  CONSTRAINT fk_tb_curriculos_tb_cursos1
    FOREIGN KEY (id_curso)
    REFERENCES tb_cursos (id_curso)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_curriculo_disciplina
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_curriculo_disciplina ;

CREATE TABLE tb_curriculo_disciplina (
  id_curriculo INT NOT NULL,
  id_disciplina INT NOT NULL,
  PRIMARY KEY (id_curriculo, id_disciplina) ,
  INDEX fk_tb_curriculo_disciplina_tb_disciplinas1_idx (id_disciplina ASC) ,
  CONSTRAINT fk_tb_curriculo_disciplina_tb_curriculos1
    FOREIGN KEY (id_curriculo)
    REFERENCES tb_curriculos (id_curriculo)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_tb_curriculo_disciplina_tb_disciplinas1
    FOREIGN KEY (id_disciplina)
    REFERENCES tb_disciplinas (id_disciplina)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_bibliografia_livro
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_bibliografia_livro ;

CREATE TABLE tb_bibliografia_livro (
  id_livro INT NOT NULL,
  id_bibliografia INT NOT NULL,
  PRIMARY KEY (id_livro, id_bibliografia) ,
  INDEX fk_tb_bibligrafia_acervo_tb_bibliografias1_idx (id_bibliografia ASC) ,
  CONSTRAINT fk_tb_bibligrafia_acervo_tb_acervo1
    FOREIGN KEY (id_livro)
    REFERENCES tb_livro (id_livro)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_tb_bibligrafia_acervo_tb_bibliografias1
    FOREIGN KEY (id_bibliografia)
    REFERENCES tb_bibliografias (id_bibliografia)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Tabletb_livro_favorito
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_livro_favorito ;

CREATE TABLE tb_livro_favorito (
  id_livro_favorito INT NOT NULL AUTO_INCREMENT,
  id_livro INT NOT NULL,
  id_professor INT NOT NULL,
  PRIMARY KEY (id_livro_favorito, id_livro, id_professor) ,
  INDEX fk_tb_livro_favorito_tb_professores1_idx (id_livro ASC, id_professor ASC) ,
  CONSTRAINT fk_tb_livro_favorito_tb_livro1
    FOREIGN KEY (id_livro)
    REFERENCES tb_livro (id_livro)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_tb_livro_favorito_tb_professores1
    FOREIGN KEY (id_professor)
    REFERENCES tb_professores (id_professor)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION);

  -- -----------------------------------------------------
-- Tabletb_cursos_disciplinas
-- -----------------------------------------------------
-- DROP TABLE IF EXISTStb_cursos_disciplinas;
CREATE TABLE tb_cursos_disciplinas(
  id_disciplina INT NOT NULL,
  id_curso INT NOT NULL,
    FOREIGN KEY (id_disciplina)
    REFERENCES tb_disciplinas (id_disciplina)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
    FOREIGN KEY (id_curso)
    REFERENCES tb_cursos (id_curso)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;