package br.com.biblioteca.model;
/**
 * Esta classe representa a estrutura dos objetos denominados como "bibliografiaHomologada". Do ponto de vista das regras de negcios (RN), a bibliografia homologada
 *  aquela que seguiu todo o Workflow de aprovao e compra no sistema e ao final do processo torna-se vlida para determinada disciplina.
 * @author Fagundes, Emiliano.
 *
 *
 */
public class BibliografiaHomologada {

	private int idBibliografia;
	private Disciplina disciplina;
	private BibliografiaBasica basica;
	private BibliografiaComplementar complementar;
	private String dataHomologacao;

	
	public BibliografiaHomologada(int idBibliografia, BibliografiaBasica basica, BibliografiaComplementar complementar) {
		super();
		this.setIdBibliografia(idBibliografia);
		this.setBasica(basica);
		this.setComplementar(complementar);
	
	}
	
	public BibliografiaHomologada() {
		this.disciplina = new Disciplina();
		this.basica = new BibliografiaBasica();
		this.complementar = new BibliografiaComplementar();
		
	
	}
	public int getIdBibliografia(){
		return this.idBibliografia;
	}
	/**
	 * Mtodo deve verficiar nas duas colees de livros se o ID passado por parmetro j
	 * foi incluso. Retorna -1 caso no seja encontrado em nenhuma das colees.q
	 * @param idLivro
	 * @return 
	 */
	public int verificarTipoBibliografiaPorIdLivro(int idLivro){
		
		for(Livro l : this.basica.getLivros()){
			if(l.getIdLivro() == idLivro)
				return this.basica.getClassificacao();
		}
		for(Livro l : this.complementar.getLivros()){
			if(l.getIdLivro() == idLivro)
				return this.complementar.getClassificacao() ;
		}
		
		return -1; 
		
	}
	/**
	 *  Mtodo que retorna o objeto livro da ID indicada, caso no exista retorna uma nova instncia de Livro.
	 * @return
	 */
	public Livro getLivroByID(int idLivro){
		
		for(Livro l : this.basica.getLivros()){
			if(l.getIdLivro() == idLivro)
				return l;
		}
		for(Livro l : this.complementar.getLivros()){
			if(l.getIdLivro() == idLivro)
				return l;
		}
		Livro l = new Livro();
		l.setIdLivro(idLivro);
		return l;
	}
	
	/**
	 *  M todo que retorna o objeto livro da ID e classificao de bibliografia  indicada, caso no exista retorna uma nova instncia de Livro.
	 * @return
	 */
	public Livro getLivroByIDAndClassificaco(int idLivro, int classificacao){
		if(classificacao == this.basica.getClassificacao()){
			for(Livro l : this.basica.getLivros()){
				if(l.getIdLivro() == idLivro)
					return l;
			}
		}else if(classificacao == this.complementar.getClassificacao()){
			for(Livro l : this.complementar.getLivros()){
				if(l.getIdLivro() == idLivro)
					return l;
			}
		}
				
		Livro l = new Livro();
		l.setIdLivro(idLivro);
		return l;
	}
	public BibliografiaBasica getBasica() {
		return basica;
	}

	public BibliografiaComplementar getComplementar() {
		return complementar;
	}

	public String getDataHomologacao() {
		return dataHomologacao;
	}

	public void setBasica(BibliografiaBasica basica) {
		this.basica = basica;
	}

	public void setComplementar(BibliografiaComplementar complementar) {
		this.complementar = complementar;
	}

	public void setDataHomologacao(String dataHomologacao) {
		this.dataHomologacao = dataHomologacao;
	}

	public void setIdBibliografia(int idBibliografia) {
		this.idBibliografia = idBibliografia;
	}
	public boolean addBookInBasica(Livro livro){
		if( this.basica.addLivro(livro)){
			return true;
		}
		return false;
	}
	public boolean addBookInComplementar(Livro livro){
		if( this.complementar.addLivro(livro)){
			return true;
		}
		return false;
	}
	public void setDisciplina(Disciplina disciplina) {
		this.disciplina = disciplina;
	}

	@Override
	public String toString(){
		return this.getIdBibliografia() + " " +
				this.getDataHomologacao() + " " +
				this.getBasica().toString() + " " +
				this.getComplementar().toString();


	}
}
