package br.com.biblioteca.bo;

import java.sql.SQLException;
import java.text.ParseException;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import br.com.biblioteca.dao.CursoDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Curso;
import br.com.biblioteca.model.Faculdade;
import br.com.biblioteca.model.Livro;
import junit.framework.Assert;
import junit.framework.TestCase;

@RunWith(MockitoJUnitRunner.class)

public class CursoBOTest extends TestCase{

		private Curso curso;
		//private Curso curso1;
		private CursoBO cursoBO;
		
		@Mock
		CursoDAO cursoDAOMock;
		
		
		@Before
		public void setUp() throws Exception {
			cursoBO = new CursoBO();
			cursoBO.setCursoDAO(cursoDAOMock);
			
			Faculdade faculdade = new Faculdade("Facin");
			
			curso = new Curso("Engenharia de Software", "1234", faculdade);
			//curso1 = new Curso("", "", faculdade);						
		}

		
		@Test
		public void testValidarCursoCorreto() throws PersistenciaException, NegocioException {	
				boolean valido = cursoBO.validaCurso(curso);
				assertTrue(valido);
		}
		
		@Test
		public void testValidarCursoNull() throws NegocioException {
			boolean valido = cursoBO.validaCurso(null);
			assertFalse(valido);
		}
		
		@Test
		public void TestAdicionarCurso() throws PersistenciaException, NegocioException, SQLException, ParseException {	
				Mockito.when(cursoDAOMock.adicionarCurso(curso)).thenReturn(1);
				int idCurso = cursoBO.adicionarCurso(curso);
				assertEquals(true,idCurso > 0);
		}
		
		
		@Test
		public void removeCursoExistente() throws PersistenciaException, SQLException {
			int idCurso = 1;
			Mockito.when(cursoDAOMock.removerCurso(idCurso)).thenReturn(true);
			boolean excluido = cursoBO.removerCurso(idCurso);
			Assert.assertTrue(excluido);
		}

		@Test
		public void removeCursoInexistente() throws PersistenciaException, SQLException {
			int idCurso = 1;
			Mockito.when(cursoDAOMock.removerCurso(idCurso)).thenReturn(false);
			boolean excluido = cursoBO.removerCurso(idCurso);
			Assert.assertFalse(excluido);
		}
		
		@Test
		public void editarCursoValido() throws PersistenciaException, SQLException{
			Mockito.when(cursoDAOMock.editarCurso(curso)).thenReturn(true);
			boolean excluido = cursoBO.editarCurso(curso);
			Assert.assertTrue(excluido);
		}
		
		@Test
		public void editarLivroInvalido() throws PersistenciaException, SQLException{
			Mockito.when(cursoDAOMock.editarCurso(null)).thenReturn(false);
			boolean excluido = cursoBO.editarCurso(curso);
			Assert.assertFalse(excluido);
		}
		
		@Test
		public void buscaCursoPorIdExistente() throws PersistenciaException, SQLException {
			int idCurso = 1;
			Mockito.when(cursoDAOMock.BuscarCursoPorId(idCurso)).thenReturn(curso);
			Curso curso1 = cursoBO.buscarCursoPorId(idCurso);
			Assert.assertTrue(curso1 == curso);
		}
		
		@Test
		public void buscaCursoPorIdInexistente() throws PersistenciaException, SQLException {
			int idCurso = 1;
			Mockito.when(cursoDAOMock.BuscarCursoPorId(idCurso)).thenReturn(null);
			Curso curso1 = cursoBO.buscarCursoPorId(idCurso);
			Assert.assertFalse(curso == curso1);
		}
		
		
	
}
