package br.com.biblioteca.bo;



import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import br.com.biblioteca.dao.LivroDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Livro;
import br.com.biblioteca.util.MensagensConstantes;
import junit.framework.Assert;
import junit.framework.TestCase;

@RunWith(MockitoJUnitRunner.class)

public class LivroBOTest extends TestCase{

	private LivroBO livroBO;
	private Livro livroTest, livroISBNNull;
	
	@Mock
	LivroDAO livroDAOMock;
	List<Livro> livrosCorretos, livrosListaTitulo;
	
	@Before
	public void init() {
		
		livrosListaTitulo = new LinkedList<>();
		
		for (int i = 1; i <= 4; i++) {
			Livro livro = new Livro();
			livro.setAnoPublicacao(2011);
			livro.setCidadePublicacao("Porto Alegre");
			livro.setEdicao("" + i);
			livro.setEditora("EdiPUCRS");
			livro.setIdioma("Portugus");
			livro.setIsbn("12345" + i);
			livro.setTitulo("LivroTeste");
			livrosListaTitulo.add(livro);
		}
		
		livrosCorretos = new ArrayList<>();
		
		for (int i = 1; i <= 20; i++) {
			Livro livro = new Livro();
			livro.setAnoPublicacao(2011);
			livro.setCidadePublicacao("Porto Alegre");
			livro.setEdicao("1");
			livro.setEditora("EdiPUCRS");
			livro.setIdioma("Portugus");
			livro.setIsbn("12345" + i);
			livro.setTitulo("LivroTeste numero " + i);
			livrosCorretos.add(livro);
		}

		this.livroBO = new LivroBO();
		this.livroBO.setLivroDAO(livroDAOMock);
		
		livroISBNNull = new Livro();
		livroISBNNull.setAnoPublicacao(2011);
		livroISBNNull.setCidadePublicacao("Porto Alegre");
		livroISBNNull.setEdicao("1");
		livroISBNNull.setEditora("EdiPUCRS");
		livroISBNNull.setIdioma("Portugus");
		livroISBNNull.setIsbn(null);
		livroISBNNull.setTitulo("LivroTesteSBNNull");
		
		livroTest = new Livro();
		livroTest.setAnoPublicacao(2011);
		livroTest.setCidadePublicacao("Porto Alegre");
		livroTest.setEdicao("1");
		livroTest.setEditora("EdiPUCRS");
		livroTest.setIdioma("Portugus");
		livroTest.setIsbn("ISBN");
		livroTest.setTitulo("LivroTesteSBNNull");
	}
	
	@Test
	public void testValidarLivroCorreto() throws PersistenciaException, NegocioException {	
			boolean valido = livroBO.validaLivro(livroTest);
			assertTrue(valido);
	}
	
	@Test
	public void testValidarLivroNull() throws NegocioException {
		livroBO.validaLivro(null);
	}
	
	@Test
	public void testValidarLivroISBNNull() throws NegocioException {
		livroTest.setIsbn(null);
		livroBO.validaLivro(livroTest);
	}
	
	@Test
	public void testListarLivrosPeloTitulo() throws NegocioException, PersistenciaException, SQLException{
		String titulo = "LivroTeste";
		List<Livro> livros = new LinkedList<>();
		Mockito.when(livroDAOMock.listarLivros(titulo)).thenReturn((LinkedList<Livro>) livrosListaTitulo);
		
		livros = livroBO.listarLivros(titulo);
		
		assertTrue(livros.size() == 4);
	}
	
	@Test
	public void testMensagemCadastrarLivro() throws PersistenciaException, SQLException, ParseException{
		Mockito.when(livroDAOMock.cadastraLivro(livroISBNNull)).thenThrow(new PersistenciaException(
				MensagensConstantes.MSG_ERR_LIVRO_DADOS_INVALIDOS.replace("?", livroISBNNull.getTitulo())));
		try {
			livroBO.cadastraLivro(livroISBNNull);
			fail("Livro no devia ser cadastrado");
		} catch (Exception e) {
			Assert.assertTrue("Mensagem de erro: Teste cadastrar livro falhou", e.getMessage().equals(MensagensConstantes.MSG_ERR_LIVRO_DADOS_INVALIDOS.replace("?", livroISBNNull.getTitulo())));
		}
	}
	
//	@Test
//	public void testCadastrarLivroValido() throws PersistenciaException, SQLException, ParseException{
//		Mockito.when(livroDAOMock.cadastraLivro(livroTest)).thenReturn(1);
//		int idLivro = livroBO.cadastraLivro(livroTest);
//		Assert.assertTrue(idLivro > 0);
//	}
	
	@Test
	public void buscaLivrobyTituloExistente() throws PersistenciaException, SQLException {
		String titulo = "LivroTeste";
		Mockito.when(livroDAOMock.buscaLivrobyTitulo(titulo)).thenReturn(livrosListaTitulo.get(0));
		Livro livro = livroBO.buscaLivrobyTitulo(titulo);
		Assert.assertTrue(livrosListaTitulo.get(0) == livro);
	}
	
	@Test
	public void buscaLivrobyTituloInexistente() throws PersistenciaException, SQLException {
		String titulo = "LivroTeste";
		Mockito.when(livroDAOMock.buscaLivrobyTitulo(titulo)).thenReturn(null);
		Livro livro = livroBO.buscaLivrobyTitulo(titulo);
		Assert.assertFalse(livrosListaTitulo.get(0) == livro);
	}
	
	@Test
	public void buscaLivrobyIdExistente() throws PersistenciaException, SQLException {
		int idLivro = 1;
		Mockito.when(livroDAOMock.buscaLivrobyId(idLivro)).thenReturn(livrosListaTitulo.get(0));
		Livro livro = livroBO.buscaLivrobyId(idLivro);
		Assert.assertTrue(livrosListaTitulo.get(0) == livro);
	}
	
	@Test
	public void buscaLivrobyIdInexistente() throws PersistenciaException, SQLException {
		int idLivro = 1;
		Mockito.when(livroDAOMock.buscaLivrobyId(idLivro)).thenReturn(null);
		Livro livro = livroBO.buscaLivrobyId(idLivro);
		Assert.assertFalse(livrosListaTitulo.get(0) == livro);
	}
	
	@Test
	public void removeLivroExistente() throws PersistenciaException, SQLException {
		int idLivro = 1;
		Mockito.when(livroDAOMock.removeLivro(idLivro)).thenReturn(true);
		boolean excluido = livroBO.removeLivro(idLivro);
		Assert.assertTrue(excluido);
	}

	@Test
	public void removeLivroInexistente() throws PersistenciaException, SQLException {
		int idLivro = 1;
		Mockito.when(livroDAOMock.removeLivro(idLivro)).thenReturn(false);
		boolean excluido = livroBO.removeLivro(idLivro);
		Assert.assertFalse(excluido);
	}
	
	@Test (expected=NegocioException.class)
	public void editarLivroValido() throws PersistenciaException, SQLException, NegocioException, ParseException{
		int idLivro = 1;
		Mockito.when(livroDAOMock.editarLivro(livroTest)).thenReturn(true);
		boolean excluido = livroBO.editarLivro(livroTest);
		Assert.assertTrue(excluido);
	}
	
	@Test (expected=NegocioException.class)
	public void editarLivroInvalido() throws PersistenciaException, SQLException, NegocioException, ParseException{
		int idLivro = 1;
		Mockito.when(livroDAOMock.editarLivro(livroTest)).thenReturn(false);
		boolean excluido = livroBO.editarLivro(livroTest);
		Assert.assertFalse(excluido);
	}
}
