package br.com.biblioteca.bo;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import br.com.biblioteca.dao.UsuarioDAO;
import br.com.biblioteca.exception.NegocioException;
import br.com.biblioteca.exception.PersistenciaException;
import br.com.biblioteca.model.Usuario;
import br.com.biblioteca.util.MensagensConstantes;
import junit.framework.TestCase;

@RunWith(MockitoJUnitRunner.class)

public class UsuarioBOTest extends TestCase {

	private UsuarioBO usuarioBO;
	private Usuario usuario;

	@Mock
	UsuarioDAO usuarioMock;

	@Before
	public void init() {
		this.usuarioBO = new UsuarioBO();
		this.usuarioBO.setUsuarioDAO(usuarioMock);
		this.usuario = new Usuario();
		this.usuario.setIdUsuario(1);
		this.usuario.setEmail("admin");
		this.usuario.setSenha("admin");
	}

	@Test
	public void testValidarUsuarioCorreto() throws PersistenciaException, NegocioException {

		Usuario userParameter = new Usuario();
		userParameter.setEmail("admin");
		userParameter.setSenha("admin");

		Usuario userReturn = new Usuario();
		userReturn.setEmail("admin");
		userReturn.setIdUsuario(1);
		userReturn.setNome("admin");

		Mockito.when(usuarioMock.validarUsuario(userParameter)).thenReturn(userReturn);

		Usuario user = usuarioBO.validaUsuario(userParameter);

		assertEquals(userReturn, user);

	}

	@Test
	public void testValidarUsuarioRetornoSemSenha() throws PersistenciaException, NegocioException {

		Usuario userParameter = new Usuario();
		userParameter.setEmail("admin");
		userParameter.setSenha("admin");

		Usuario userReturn = new Usuario();
		userReturn.setEmail("admin");
		userReturn.setIdUsuario(1);
		userReturn.setNome("admin");

		Mockito.when(usuarioMock.validarUsuario(userParameter)).thenReturn(userReturn);

		Usuario user = usuarioBO.validaUsuario(userParameter);

		assertEquals(user.getSenha(), null);

	}

	@Test(expected = NegocioException.class)
	public void testValidarUsuarioSemEmail() throws NegocioException, PersistenciaException {

		Usuario userParameter = new Usuario();
		userParameter.setEmail("");
		userParameter.setSenha("admin");

		Mockito.when(usuarioMock.validarUsuario(userParameter)).thenReturn(null);

		this.usuarioBO.validaUsuario(userParameter);
	}

	@Test(expected = NegocioException.class)
	public void testValidarUsuarioSemSenha() throws NegocioException, PersistenciaException {

		Usuario userParameter = new Usuario();
		userParameter.setEmail("admin");
		userParameter.setSenha("");

		Mockito.when(usuarioMock.validarUsuario(userParameter)).thenReturn(null);

		this.usuarioBO.validaUsuario(userParameter);
	}

	@Test(expected = NegocioException.class)
	public void testExceptionBarrier() throws NegocioException, PersistenciaException {
		Usuario userParameter = new Usuario();
		userParameter.setEmail("");
		userParameter.setSenha("admin");

		Mockito.when(usuarioMock.validarUsuario(userParameter)).thenThrow(new PersistenciaException("Teste Exception"));

		this.usuarioBO.validaUsuario(userParameter);
	
	}
	
	@Test
	public void testNegocioExceptionMsgPersistenciaException() throws PersistenciaException {

		Usuario userParameter = new Usuario();
		userParameter.setEmail("");
		userParameter.setSenha("admin");

		Mockito.when(usuarioMock.validarUsuario(userParameter)).thenThrow(new PersistenciaException("Teste Exception"));

		String exceptionMsg = MensagensConstantes.SYSTEM_GENERIC_ERROR;
		
		try {
			this.usuarioBO.validaUsuario(userParameter);
		} catch (NegocioException e) {
			assertEquals(exceptionMsg, e.getMessage());
		}

	}
	
	@Test
	public void testNegocioExceptionMsgUsuarioInvalido() throws PersistenciaException {

		Usuario userParameter = new Usuario();
		userParameter.setEmail("");
		userParameter.setSenha("admin");

		Mockito.when(usuarioMock.validarUsuario(userParameter)).thenReturn(null);

		String exceptionMsg = MensagensConstantes.MSG_ERR_USUARIO_SENHA_INVALIDOS;
		
		try {
			this.usuarioBO.validaUsuario(userParameter);
		} catch (NegocioException e) {
			assertEquals(exceptionMsg, e.getMessage());
		}

	}

}
